//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//---------------------------------------------------------------------------
//---------------------------------------------------------------------------


#ifndef _DRIVER_H_
#define _DRIVER_H_

#ifdef __cplusplus
extern"C"{
#endif

/////////////////////////////////////////////////////////////////////////////
#include "../src/ddf/ddf_type.h"
#include "../src/ddf/ddf_val.h"
#include "../src/swi.h"
#include "../src/ddf/ioctl.h"
#include "../src/ddf/uid.h"
/////////////////////////////////////////////////////////////////////////////

#include "../src/mini2440/driver/device_type.h"
#include "../src/mini2440/driver/device_id.h"





__swi(DDF) INT _swi_ddf_device_io_control(
                            UINT            ioctl,
                    const   DEVICE_OBJ *    pDevice,
                    const   BUFFER_OBJ *    pBuffer_in,
                            BUFFER_OBJ *    pBuffer_out
                    );
                    
__swi(DDF) UID _swi_ddf_device_open(
                    UINT            ioctl,
                    DEVICE_OBJ *    pDevice,
                    UINT            major, 
                    UINT            minor
                    );

__swi(DDF) INT _swi_ddf_device_close(
                            UINT           ioctl,
                    const   DEVICE_OBJ *   pDevice,
                            UINT           major,
                            UINT           minor
                    );
        
        
        

// *** ddf_DeviceOpen ***
// -- Description --
//  Open a device
// -- Parameters --
//  "a", a potiner of DEVICE_OBJ
//  "b", major device_id
//  "c", minor device_id
// -- Return --
//  DDF_DEVICE_UNKNOWN, DDF_DEVICE_IN_USE
//  The returned values were predefined in ddf_val.h
// -- Usage --
//  i.e.
//  ...
//  DEVICE_OBJ com1;    //you must define a DEVICE_OBJ,
                        //then send its address to this function
//  UID ret_val = ddf_DeviceOpen(&com1, SERIAL, COM1);
//  switch(ret_val)
//  {
//  case DDF_SUCCESS:
//     ...
//     break;
//  case DDF_DEVICE_IN_USE:
//     ...
//     break;
//  case DDF_DEVICE_UNKNOWN:
//     ...
//     break;
//  }
//  ...
#define ddf_DeviceOpen(a, b, c) \
            _swi_ddf_device_open((DDF_DEVICE_OPEN), (a), (b), (c))
            
            
            
            
            
            
// *** ddf_DeviceClose ***
// -- Description --
//  Close a Device
// -- Parameters --
//  "a", a pointer of a device which will be closed
//  "b", major device code, could be zero
//  "c", minor device code 
// -- Return --
//  DDF_SUCCESS, DDF_FAILURE
// -- Usage --
//  i.e.
//  ...
//  DEVICE_OBJ dev;
//  ddf_DeviceOpen(&dev, MAJOR, MINOR);
//  ... 
//  ddf_DeviceClose(&dev, MAJOR, MINOR);
//  ...
#define ddf_DeviceClose(a, b, c) \
            _swi_ddf_device_close((DDF_DEVICE_CLOSE), (a), (b), (c))
            
            
            
            
                        

// *** ddf_device_io_control ***
// -- Description -- 
//  Send a IOCTL code directly to a specified device driver causing 
//  the corresponding device to perform the specified operation
// -- Parameters --
// "a", ioctl code
// "b", a pointer of a device
// "c", buffer in
// "d", buffer out
// -- Return -- 
//  DDF_SUCCESS, DDF_FAILURE ...
//  The returned values were predefined in ddf_val.h
// -- Usage --
//  i.e.
//  ...
//  DEVICE_OBJ com1;
//  BUFFER_OBJ str;
//  str.buffer = "Use com1 to send a string";
//  ddf_DeviceOpen(&com1, SERIAL, COM1);
//  ddf_DeviceIoControl(IOCTL_SERIAL_SEND, com1, &str, 0);
//  ddf_DeviceClose(&com1);
//  ...
#define ddf_DeviceIoControl(a, b, c, d) \
            _swi_ddf_device_io_control((a), (b), (c), (d))
            

              
               
            
#ifdef __cplusplus
}
#endif

#endif
            