//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
#include "../inc/typedef.h"
#include "../swi.h"
#include "./uid.h"
#include "./ddf_type.h"
#include "./ddf_val.h"
#include "./ddf_core.h"
#include "./ioctl.h"
#include <string.h>

//When a application raise a swi to request ddf service 
//the related parmeters will be sent to this function for 
//a specified service
INT _ddf_request_dispatch(PUINT pArgs)
{
    UINT ioctl = pArgs[0];
    PDEVICE_OBJ pDevice = (PDEVICE_OBJ)pArgs[1];
    
    UINT major = pArgs[2];
    UINT minor = pArgs[3]; 
    
    PBUFFER_OBJ pBuffer_in = (PBUFFER_OBJ)pArgs[2];
    PBUFFER_OBJ pBuffer_out = (PBUFFER_OBJ)pArgs[3];
    
    UINT ret_val;
    
    //enter_sys_mode();
    
    switch(ioctl)
    {
    case DDF_DEVICE_OPEN:
        ret_val = _ddf_device_open(pDevice, major, minor);
        break;
    case DDF_DEVICE_CLOSE:
        ret_val = _ddf_device_close(pDevice, major, minor);
        break;
    default:
        ret_val = _ddf_device_io_control(
                            ioctl, 
                            pDevice, 
                            pBuffer_in, 
                            pBuffer_out);
    }
    
    //exit_sys_mode();
    
    return ret_val;
}

INT _ddf_init(DEVICE_OBJ * pDevice)
{
    //some additional initiliation add here ...
    
    return _devices_init(pDevice);
}

INT _devices_init(DEVICE_OBJ * pDevice)
{
    PDEVICE_OBJ pDev = _devices_register;
    //Initilize every device which was resgitered in device_list
    while(pDev->device_id){
        if(pDev->init() == DDF_FAILURE){
            //If any initialization failed, return failure
            pDevice->device_id = pDev->device_id;
            return DDF_FAILURE;
        }
        pDev++;
    }
    //Everything is done
    return DDF_SUCCESS;
}

UID _ddf_device_open(DEVICE_OBJ * pDevice, UINT major, UINT minor)
{
    PDEVICE_OBJ pDev = _devices_register;
    UID uid; 
    
    while(pDev->device_id){
        if(pDev->device_id == major){
            uid = pDev->open(major, minor);
            if(uid > 0){
                //uid>0 means current device is idle and ready to work
                //so return a repetition of current device object which 
                //is containg the entry of driver 
                strcpy(pDevice->name, pDev->name);
                pDevice->device_id = pDev->device_id;
                pDevice->init = pDev->init;
                pDevice->open = pDev->open;
                pDevice->close = pDev->close;
                pDevice->driver_entry = pDev->driver_entry; 
                //Return the valid uid
                return uid;
            }else if(uid == DDF_DEVICE_IN_USE){
                return DDF_DEVICE_IN_USE;
            }
        }
        pDev++;
    }
    //No device id match the major and minor code, so return unknown device
    return DDF_DEVICE_UNKNOWN;
}

INT _ddf_device_close(const DEVICE_OBJ * pDevice, UINT major, UINT minor)
{
    return pDevice->close(major, minor);
}

INT _ddf_device_io_control(
                UINT            ioctl,
        const   DEVICE_OBJ *    pDev,
        const   BUFFER_OBJ *    pBuffer_in,
                BUFFER_OBJ *    pBuffer_out
        )
{
    return pDev->driver_entry(ioctl, pBuffer_in, pBuffer_out);
}

//_ddf_driver_dispatch is a driver development api 
//which will be called in driver_entry of device driver 
INT _ddf_driver_dispatch(
        const   IO_REQ *        pIo_request_map,
                UINT            ioctl,
        const   BUFFER_OBJ *    pBuffer_in,
                BUFFER_OBJ *    pBuffer_out
        )
{
    //Is there a matched ioctl service routine was registered in device 
    //driver io request map? 
    //  if yes, call ioctl service routine 
    //  if not, return DDF_IOCTL_UNSUPPORT
    while(pIo_request_map->ioctl){
        if(pIo_request_map->ioctl == ioctl)
            return pIo_request_map->driver_request(pBuffer_in, pBuffer_out);
        pIo_request_map++;
    }
    return DDF_IOCTL_UNSUPPORT;
}

