//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
#ifndef _DFF_TYPE_H_
#define _DFF_TYPE_H_

#ifdef __cplusplus
extern"C"{
#endif


// -- Structure -- 
// "buffer_obj", alias "BUFFER_OBJ", pointer "PBUFFER_OBJ"
// -- Description -- 
// point to buffer for data transmission
// -- Members --   
// 1) "pbuffer" , buffer pointer
// 2) "size" , sizeof(buffer)              
typedef struct _buffer_obj{
    void * pbuffer;
    UINT size;
}BUFFER_OBJ, *PBUFFER_OBJ;


// -- Structrue -- 
// "device_object", alias "DEVICE_OBJ", pointer "PDEVICE_OBJ"
// -- Description -- 
// device abstract, store information concerning the device
// and entry of driver for io service
// -- Members --
// 1) "name" , device name in a null terminated string
// 2) "device_id", a macro stand for the device , i.e. "MINI2440_SERIAL"
// 3) "init", pointer of a function for initilize a device
// 4) "open", pointer of a function for open a device
// 5) "close", pointer of a function for close a device
// 6) "driver_entry", pointer of driver entry
typedef struct _device_object{
    CHAR name[20];
    UINT device_id;
    INT (*init)(void);
    UID (*open)(UINT, UINT);
    INT (*close)(UINT, UINT);
    INT (*driver_entry)(UINT, const BUFFER_OBJ *, BUFFER_OBJ *);  
}DEVICE_OBJ, *PDEVICE_OBJ;


// -- Structure --
// "io_request", alias "IO_REQ", pointer "PIO_REQ"
// -- Description --
// a ioctl code relate to a driver service routine
// -- Members --
// 1) "ioctl", IO control code 
// 2) "driver_request", point to a ioctl specified driver service routine
typedef struct _io_request{
    UINT ioctl;
    INT (*driver_request)(const BUFFER_OBJ *, BUFFER_OBJ *);
}IO_REQ, *PIO_REQ;


#ifdef __cplusplus
}
#endif

#endif