//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
// 
//--------------------------------------------------------------------------
#include "../../inc/typedef.h"
#include "./draw_lib.h"
#include "./font_lib.h"
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

static TEXT_PRINT_STYLE _sys_text_print_style = {0, WHITE, WHITE, 0, 0, 0, 0};

static PTEXT_CLASS _psys_text_class;
static PTEXT_CLASS _psys_text_class_little;
static PTEXT_CLASS _psys_text_class_big;

void _font_lib_init(void)
{
    _psys_text_class_little = (PTEXT_CLASS)malloc(sizeof(TEXT_CLASS));
    _psys_text_class_big = (PTEXT_CLASS)malloc(sizeof(TEXT_CLASS));
    
    _psys_text_class_little->pascii_font = &_fonts_register[0];
    _psys_text_class_little->pchinese_font = &_fonts_register[2];
    _psys_text_class_little->ptext_print_style = &_sys_text_print_style;
    
    _psys_text_class_big->pascii_font = &_fonts_register[1];
    _psys_text_class_big->pchinese_font = &_fonts_register[3];
    _psys_text_class_big->ptext_print_style = &_sys_text_print_style;
    
    _psys_text_class = _psys_text_class_little;
}

void _lcd_printf(
            INT               x_pos, 
            INT               y_pos,
    const   TEXT_CLASS       * ptext_class,       
    const   CHAR             * fmt, 
            ...
    )
{
/* 
    UINT ascii_char_width = ptext_class->pascii_font->char_width;
    UINT ascii_char_height = ptext_class->pascii_font->char_height;
    
    UINT chinese_char_width = ptext_class->pchinese_font->char_width;
    UINT chinese_char_height = ptext_class->pchinese_font->char_height;
*/    
    CHAR _lcd_print_buffer[LCD_PRINT_BUFFER_LENGTH_MAX];
    PCHAR pstr = _lcd_print_buffer;
    UINT length = 0;
    
    va_list ap;
    va_start(ap, fmt);
    vsprintf(_lcd_print_buffer, fmt, ap);
    va_end(ap);
/*
    if(*pstr > 0x7f && *(pstr + 1) > 0x7f){
        if(x_pos + chinese_char_width > X_SIZE 
            || y_pos + chinese_char_height > Y_SIZE)
                eturn;    
    }else{
        if(x_pos + ascii_char_width > X_SIZE 
            || y_pos + ascii_char_height > Y_SIZE)
                return;
    }  
*/
    while(*pstr != 0 && length < LCD_PRINT_BUFFER_LENGTH_MAX){        

        if(*pstr != '\n'){
            if(*pstr > 0x7f && *(pstr + 1) > 0x7f){    
/*                
                if(x_pos + chinese_char_width > X_SIZE 
                    || y_pos + chinese_char_height > Y_SIZE)
                    return;    
*/                  
                _lcd_put_char_chinese(&x_pos, &y_pos, ptext_class, pstr);

                pstr += 2;
                length += 2;
            }else{
/*            
                if(x_pos + ascii_char_width > X_SIZE 
                    || y_pos + ascii_char_height > Y_SIZE)
                    return;
*/                    
                _lcd_put_char_ascii(&x_pos, &y_pos, ptext_class, pstr);
                
                pstr += 1;
                length += 1;
            }
        }else{
            //ignore newline 
            pstr += 1;
            length += 1;
        }
    }
}

void _lcd_put_char_ascii(
            PINT              px_pos, 
            PINT              py_pos, 
    const   TEXT_CLASS       * ptext_class,
    const   CHAR             * pchar
    )
{
    UINT char_width = ptext_class->pascii_font->char_width;
    UINT char_size = ptext_class->pascii_font->char_size;
    const UINT * pchar_map = ptext_class->pascii_font->pchar_map;
    UINT front_color = ptext_class->ptext_print_style->front_color;
    UINT back_color;
    UINT is_set_back_color 
            = ptext_class->ptext_print_style->is_set_back_color;
    UINT space = ptext_class->ptext_print_style->space;
    
    UINT char_start_offset = *pchar * char_size >> 2;
    UINT fetch_times_max = char_size >> 2;
    UINT fetch_times;
    UINT char_data;
    UINT x, y;
    UINT mask;
    
    //optimized for speed
    if(is_set_back_color){
        
        back_color = ptext_class->ptext_print_style->back_color;
        
        for(fetch_times = 0; fetch_times < fetch_times_max; fetch_times++){
        
            //fetch 4 bytes every time
            char_data = pchar_map[char_start_offset++];
            
            for(y = fetch_times << 2; y < (fetch_times + 1) << 2; y++){
                mask = BIT7;
                
                for(x = 0; x < char_width; x++){
                    if(mask & char_data)
                        _put_pixel(*px_pos + x, *py_pos + y, front_color);
                    else
                        _put_pixel(*px_pos + x, *py_pos + y, back_color);
                        
                    mask >>= 1;
                }
                
                for(x = char_width; x < char_width + space; x++)
                    _put_pixel(*px_pos + x, *py_pos + y, back_color);    
                    
                char_data >>= 8;
            }
        }
    }else{
        for(fetch_times = 0; fetch_times < fetch_times_max; fetch_times++){
        
            //fetch 4 bytes every time
            char_data = pchar_map[char_start_offset++];
            
            for(y = fetch_times << 2; y < (fetch_times + 1) << 2; y++){
                mask = BIT7;
                
                for(x = 0; x < char_width; x++){
                    if(mask & char_data)
                        _put_pixel(*px_pos + x, *py_pos + y, front_color);
                        
                    mask >>= 1;
                }
                
                char_data >>= 8;
            }
        }
    }     
    
    (*px_pos) += char_width + space;
}

void _lcd_put_char_chinese(
            PINT              px_pos,
            PINT              py_pos,
    const   TEXT_CLASS       * ptext_class,
    const   CHAR             * pchar
    )
{
    UINT char_width = ptext_class->pchinese_font->char_width;
    UINT char_size = ptext_class->pchinese_font->char_size;
    const UINT * pchar_map = ptext_class->pchinese_font->pchar_map;
    UINT front_color = ptext_class->ptext_print_style->front_color;
    UINT back_color; 
    UINT is_set_back_color 
            = ptext_class->ptext_print_style->is_set_back_color;
    UINT space = ptext_class->ptext_print_style->space;  
    
    UINT char_start_offset 
            = ((pchar[0] - 0xa1) * 94 + (pchar[1] - 0xa1)) * char_size >> 2;
    UINT fetch_times_max = char_size >> 2;
    UINT fetch_times;
    UINT char_data;
    UINT x, y;
    UINT mask;
    
    //optimized for speed
    if(char_width == 12){
        if(is_set_back_color){
        
            back_color = ptext_class->ptext_print_style->back_color;
            
            for(fetch_times = 0; fetch_times < fetch_times_max; fetch_times++){
            
                //fetch 4 bytes every time
                char_data = pchar_map[char_start_offset++];
                
                for(y = fetch_times << 1; y < (fetch_times + 1) << 1; y++){
                    mask = BIT7;
                    for(x = 0; x < 8; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                        else
                            _put_pixel(*px_pos + x, *py_pos + y, back_color);
                            
                        mask >>= 1;
                    }
                    
                    mask = BIT15;
                    for(x = 8; x < 12; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                        else
                            _put_pixel(*px_pos + x, *py_pos + y, back_color);
                            
                        mask >>= 1;
                    }
                    
                    for(x = char_width; x < char_width + space; x++)
                        _put_pixel(*px_pos + x, *py_pos + y, back_color); 
                    
                    char_data >>= 16;
                }
            }
        }else{
            for(fetch_times = 0; fetch_times < fetch_times_max; fetch_times++){
            
                //fetch 4 bytes every time
                char_data = pchar_map[char_start_offset++];
                
                for(y = fetch_times << 1; y < (fetch_times + 1) << 1; y++){
                    mask = BIT7;
                    for(x = 0; x < 8; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                            
                        mask >>= 1;
                    }
                    
                    mask = BIT15;
                    for(x = 8; x < 12; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                            
                        mask >>= 1;
                    }
                    
                    char_data >>= 16;
                }
            }
        }
    }else if(char_width == 16){
        if(is_set_back_color){
            for(fetch_times = 0; fetch_times < fetch_times_max; fetch_times++){
            
                //fetch 4 bytes every time
                char_data = pchar_map[char_start_offset++];
                
                for(y = fetch_times << 1; y < (fetch_times + 1) << 1; y++){
                    mask = BIT7;
                    for(x = 0; x < 8; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                        else
                            _put_pixel(*px_pos + x, *py_pos + y, back_color);
                            
                        mask >>= 1;
                    }
                    
                    mask = BIT15;
                    for(x = 8; x < 16; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                        else
                            _put_pixel(*px_pos + x, *py_pos + y, back_color);
                            
                        mask >>= 1;
                    }
                    
                    for(x = char_width; x < char_width + space; x++)
                        _put_pixel(*px_pos + x, *py_pos + y, back_color); 
                        
                    char_data >>= 16;
                }

            }
        }else{
            for(fetch_times = 0; fetch_times < fetch_times_max; fetch_times++){
            
                //fetch 4 bytes every time
                char_data = pchar_map[char_start_offset++];
                
                for(y = fetch_times << 1; y < (fetch_times + 1) << 1; y++){
                    mask = BIT7;
                    for(x = 0; x < 8; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                            
                        mask >>= 1;
                    }
                    
                    mask = BIT15;
                    for(x = 8; x < 16; x++){
                        if(mask & char_data)
                            _put_pixel(*px_pos + x, *py_pos + y, front_color);
                            
                        mask >>= 1;
                    }
                    
                    char_data >>= 16;
                }
            }
        }    
    }
        
    (*px_pos) += char_width + space;
}

const TEXT_CLASS * _font_set(UINT size_pt, const CHAR *name)
{
    //"name" would be used in the future
    
    const TEXT_CLASS * pold_sys_text_class = _psys_text_class;
    
    if(size_pt == 12){
        _psys_text_class = _psys_text_class_little;
        return pold_sys_text_class;
    }else if(size_pt == 16){
        _psys_text_class = _psys_text_class_big;
        return pold_sys_text_class;
    }else{
        return 0;
    }
}

UINT _char_horizontal_space_set(UINT space_pt)
{
    UINT old_space = _psys_text_class->ptext_print_style->space;
    _psys_text_class->ptext_print_style->space = space_pt;
    return old_space;
}

UINT _char_front_color_set(UINT front_color)
{
    UINT old_front_color = _psys_text_class->ptext_print_style->front_color;
    _psys_text_class->ptext_print_style->front_color = front_color;
    return old_front_color;
}

UINT _char_back_color_set(UINT back_color)
{
    UINT old_back_color = _psys_text_class->ptext_print_style->back_color;
    _psys_text_class->ptext_print_style->back_color = back_color;
    return old_back_color;    
}

void _char_back_color_enable(void)
{
    _psys_text_class->ptext_print_style->is_set_back_color = 1;
}

void _char_back_color_disable(void)
{
    _psys_text_class->ptext_print_style->is_set_back_color = 0;
}

PTEXT_CLASS _sys_text_class_get(void)
{
    return _psys_text_class;
}

void _font_lib_shut_down(void)
{
}




