//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
#include "../../inc/typedef.h"
#include "../mini2440/target/S3C2440_regs.h"
#include "../../inc/driver.h"
#include "../../inc/task.h"

typedef struct _task{
     UID pid;                   //offset +0
    UINT stack_length;          //offset +4
    UINT prio;                  //offset +8
    UINT registers_frame[17];   //offset +12
    UINT CS;                    //offset +80
}TASK, *PTASK;

static TASK _tasks_register[TASK_NUMBER_MAX];

PTASK _pcurrent_task = _tasks_register + 1;
PTASK _pnext_task = _tasks_register + 1;
PTASK _plast_task = _tasks_register;

void context_switch(void);
void sp_update(UINT sp_val);

// *** scheduler ***
// -- Descprition --
// This function detemine next running task,
// round-robin algorithm, very very... simple
// -- Parameters --
//      None
// -- Return --
//      None
__inline void scheduler(void)
{
    if(_pcurrent_task == _plast_task){
        _pnext_task = _tasks_register + 1;
    }else{
        _pnext_task++;
    }
}

// *** kernel_tick ***
// -- Descprition --
//  ...

void kernel_tick(void *param)
{
    clean_pending((PINT_REG)param);
        
    scheduler();
    
    context_switch();
}

UID task_create(
            TASK_ROUTINE        task_routine, 
            void              * arg, 
            UINT              * stack, 
            UINT                stack_length, 
            UINT                prio
            )
{
    _plast_task++;
    
    _plast_task->pid = uid_generate();
    _plast_task->stack_length = stack_length;
    _plast_task->prio = prio;
    _plast_task->registers_frame[0] = (UINT)arg;
    _plast_task->registers_frame[13] = (UINT)stack;
    _plast_task->registers_frame[16] = (UINT)task_routine;
    
    return _plast_task->pid;
}

UINT os_start(void)
{
    UINT sp_val 
        = _tasks_register[1].registers_frame[13];
    void * parg 
        = (void *)_tasks_register[1].registers_frame[0];
    TASK_ROUTINE first_task 
        = (TASK_ROUTINE)(_tasks_register[1].registers_frame[16]);
    
    TIMER_PARAM timer_param;
    BUFFER_OBJ buf_in;
    
    DEVICE_OBJ ints_ctrl, timer;
    
    ddf_DeviceOpen(&ints_ctrl, INTs, NULL);
    ddf_DeviceOpen(&timer, TIMERs, NULL);
    
    //configure timer 4
    timer_param.id = 4;     
    
    //kernel tick will occur at every 1ms
    timer_param.param = 1;  
    
    buf_in.pbuffer = &timer_param;
    buf_in.size = sizeof(TIMER_PARAM);
    
    ddf_DeviceIoControl(
            IOCTL_TIMER_TIME_SET, 
            &timer, 
            &buf_in, 
            NULL);
    
    //enable timer 4 and timer 4 interrupt
    timer_param.param = ON;

    ddf_DeviceIoControl(
            IOCTL_TIMER_INT_ENABLE, 
            &ints_ctrl, 
            (PBUFFER_OBJ)4, 
            NULL);
            
    ddf_DeviceIoControl(
            IOCTL_TIMER_ON_OFF, 
            &timer, 
            &buf_in, 
            NULL);
    
    sp_update(sp_val);
  
    first_task(parg);
    
    return 1;
}

UINT os_stop(void)
{
    return 1;
}

UINT os_exit(void)
{
    return 1;
}

UINT task_sleep(UID PID, UINT ms)
{
    return 1;
}

UINT task_wake_up(UID PID)
{
    return 1;
}


UINT task_exit(UID PID)
{
    return 1;
}

UINT enter_critical(VOID)
{
    DEVICE_OBJ ints;
    
    ddf_DeviceOpen(&ints, INTs, 0);
    
    ddf_DeviceIoControl(
        IOCTL_CPU_IRQ_DISABLE, 
        &ints,
        NULL, 
        NULL);
     
    return TRUE;
}

UINT exit_critical(VOID)
{
    DEVICE_OBJ ints;
    
    ddf_DeviceOpen(&ints, INTs, 0);
    
    ddf_DeviceIoControl(
        IOCTL_CPU_IRQ_ENABLE,
        &ints,
        NULL,
        NULL);
    
    return TRUE;
}