//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//---------------------------------------------------------------------------
//this file contians the SDK for fpga(Signal Acquisition And Communication Board)
//---------------------------------------------------------------------------

#include "../../../../inc/typedef.h"
#include "../../../../inc/driver.h"
#include "../../../../inc/gui.h"
#include "../../../../inc/task.h"
#include "./fpga_lib.h"

static DEVICE_OBJ fpga;

INT raise_fpga_error(INT error)
{
    //reserved for error handle in the future
    return -1;
}

INT init_fpga_lib()
{
    return ddf_DeviceOpen(&fpga, FPGA, 0);
}

INT release_fpga_lib()
{
    return ddf_DeviceClose(&fpga, 0, 0);
}

INT up_trigger(INT volt_detal, INT ch)
{
    BUFFER_OBJ buf;
    INT trigger_voltage;
    buf.pbuffer = &trigger_voltage;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_TRIGGER_VOLTAGE, &fpga, NULL, &buf);
        trigger_voltage += volt_detal;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_TRIGGER_VOLTAGE, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);   
    }
    else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHB_TRIGGER_VOLTAGE, &fpga, NULL, &buf);
        trigger_voltage += volt_detal;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_TRIGGER_VOLTAGE, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }
    
    return raise_fpga_error(ch);
}

INT set_trigger(INT volt, INT ch)
{
    BUFFER_OBJ buf_in;
    buf_in.pbuffer = &volt;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_TRIGGER_VOLTAGE, &fpga, &buf_in, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);      
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_TRIGGER_VOLTAGE, &fpga, &buf_in, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);    
    }
        
    return raise_fpga_error(ch);
}

INT get_trigger(INT ch)
{
    BUFFER_OBJ buf;
    INT trigger_voltage;
    buf.pbuffer = &trigger_voltage;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_TRIGGER_VOLTAGE, &fpga, NULL, &buf);
        return trigger_voltage;
    }
    else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHB_TRIGGER_VOLTAGE, &fpga, NULL, &buf);
        return trigger_voltage;
    }
    
    return raise_fpga_error(ch);
}

INT up_shifter(INT volt_detal, INT ch)
{
    BUFFER_OBJ buf;
    INT shifter_volt;
    buf.pbuffer = &shifter_volt;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_SHIFT_VOLTAGE, &fpga, NULL, &buf);
        shifter_volt += volt_detal;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_SHIFT_VOLTAGE, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_SHIFT_VOLTAGE, &fpga, NULL, &buf);
        shifter_volt += volt_detal;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_SHIFT_VOLTAGE, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }
    
    return raise_fpga_error(ch);
}

INT set_shifter(INT volt_detal, INT ch)
{
    BUFFER_OBJ buf;
    INT shifter_volt;
    buf.pbuffer = &shifter_volt;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_SHIFT_VOLTAGE, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_SHIFT_VOLTAGE, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }
    
    return raise_fpga_error(ch);
}

INT get_shifter(INT ch)
{
    BUFFER_OBJ buf;
    INT shifter_volt;
    buf.pbuffer = &shifter_volt;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_SHIFT_VOLTAGE, &fpga, NULL, &buf);
        return shifter_volt;
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHB_SHIFT_VOLTAGE, &fpga, NULL, &buf);
        return shifter_volt;
    }
    
    return raise_fpga_error(ch);
}

INT up_gain(INT gain_detal, INT ch)
{
    BUFFER_OBJ buf;
    INT gain;
    buf.pbuffer = &gain;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_VGA_GAIN, &fpga, NULL, &buf);
        gain += gain_detal;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_VGA_GAIN, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;        
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_VGA_GAIN, &fpga, NULL, &buf);
        gain += gain_detal;
        
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_VGA_GAIN, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;        
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }
    
    return raise_fpga_error(ch);
}

INT set_gain(INT gain, INT ch)
{
    BUFFER_OBJ buf;
    buf.pbuffer = &gain;   
     
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_VGA_GAIN, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;        
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_VGA_GAIN, &fpga, &buf, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;        
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }
    
    return raise_fpga_error(ch);    
}

INT get_gain(INT ch)
{
    BUFFER_OBJ buf;
    INT gain;
    buf.pbuffer = &gain;   
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_VGA_GAIN, &fpga, NULL, &buf);
        return gain;
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHB_VGA_GAIN, &fpga, NULL, &buf);
        return gain;
    }
    
    return raise_fpga_error(ch);
}

INT set_couple(INT DC_AC, INT ch)
{
    if(ch == CHA && DC_AC == DC_COUPLE){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_DC_COUPLE, &fpga, NULL, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }else if(ch == CHA && DC_AC == AC_COUPLE){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_AC_COUPLE, &fpga, NULL, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }else if(ch == CHB && DC_AC == DC_COUPLE){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_DC_COUPLE, &fpga, NULL, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }else if(ch == CHB && DC_AC == AC_COUPLE){
        ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
        
        ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_AC_COUPLE, &fpga, NULL, NULL);
        
        while(!(get_srcs_pending() & BIT2))
            continue;
        
        return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, &fpga, NULL, NULL);
    }
    
    return raise_fpga_error(ch | DC_AC);
}

INT get_couple(INT ch)
{
    BUFFER_OBJ buf_out;
    UINT couple;
    buf_out.pbuffer = &couple;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_COUPLE, &fpga, NULL, &buf_out);
        return couple == 0 ? DC_COUPLE : AC_COUPLE;
    }else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHB_COUPLE, &fpga, NULL, &buf_out);
        return couple == 0 ? DC_COUPLE : AC_COUPLE;        
    }
    
    return raise_fpga_error(ch);
}

INT set_pre_attenuator(INT attenuator, INT ch)
{
    if(ch == CHA)
        switch(attenuator)
        {
        case PRE_ATTENUATOR_X1:
            ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
         
            ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_PRE_ATTENUATOR_TO_X1, &fpga, NULL, NULL);
            
            while(!(get_srcs_pending() & BIT1))
                continue;
            
            return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
        
        case PRE_ATTENUATOR_X10:
            ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
         
            ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_PRE_ATTENUATOR_TO_X10, &fpga, NULL, NULL);
            
            while(!(get_srcs_pending() & BIT1))
                continue;
            
            return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
        
        case PRE_ATTENUATOR_X100:
            ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
         
            ddf_DeviceIoControl(IOCTL_FPGA_SET_CHA_PRE_ATTENUATOR_TO_X100, &fpga, NULL, NULL);
            
            while(!(get_srcs_pending() & BIT1))
                continue;
            
            return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
        
        }
    else if(ch == CHB)
        switch(attenuator)
        {
        case PRE_ATTENUATOR_X1:
            ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
         
            ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_PRE_ATTENUATOR_TO_X1, &fpga, NULL, NULL);
            
            while(!(get_srcs_pending() & BIT1))
                continue;
            
            return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
        
        case PRE_ATTENUATOR_X10:
            ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
         
            ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_PRE_ATTENUATOR_TO_X10, &fpga, NULL, NULL);
            
            while(!(get_srcs_pending() & BIT1))
                continue;
            
            return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
        
        case PRE_ATTENUATOR_X100:
            ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
         
            ddf_DeviceIoControl(IOCTL_FPGA_SET_CHB_PRE_ATTENUATOR_TO_X100, &fpga, NULL, NULL);
            
            while(!(get_srcs_pending() & BIT1))
                continue;
            
            return ddf_DeviceIoControl(IOCTL_FPGA_CLR_SRCS_HC595_DONE, &fpga, NULL, NULL);
        
        }
        
     return raise_fpga_error(attenuator);
}

INT get_pre_attenuator(INT ch)
{
    BUFFER_OBJ buf_out;
    INT attenuator;
    buf_out.pbuffer = &attenuator;
    
    if(ch == CHA){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHA_PRE_ATTENUATOR, &fpga, NULL, &buf_out);
        return attenuator;
    }
    else if(ch == CHB){
        ddf_DeviceIoControl(IOCTL_FPGA_GET_CHB_PRE_ATTENUATOR, &fpga, NULL, &buf_out);
        return attenuator;
    }
    
    return raise_fpga_error(ch);
}

BOOL is_arm_bus_select(VOID)
{
    BUFFER_OBJ buf_out;
    INT sel;
    buf_out.pbuffer = &sel;
    
    ddf_DeviceIoControl(IOCTL_FPGA_GET_HOST_ARM_SELECT, &fpga, NULL, &buf_out);
    
    return sel == 0 ? TRUE : FLASE;
}

INT host_bus_select(BOOL host_sel)
{
    BUFFER_OBJ buf_in;
    buf_in.pbuffer = &host_sel;
    
    return ddf_DeviceIoControl(IOCTL_FPGA_SET_HOST_ARM_SELECT, &fpga, &buf_in, NULL);
}


INT sample_start(VOID)
{
    return ddf_DeviceIoControl(IOCTL_FPGA_SET_SAMPLE_START, &fpga, NULL, NULL);
}

INT sample_stop(VOID)
{
    return ddf_DeviceIoControl(IOCTL_FPGA_SET_SAMPLE_STOP, &fpga, NULL, NULL);
}

BOOL is_adc_idle(VOID)
{
    BUFFER_OBJ buf_out;
    BOOL is_idle;
    
    buf_out.pbuffer = &is_idle;
    
    ddf_DeviceIoControl(IOCTL_FPGA_GET_ADC_IDLE_BUSY, &fpga, NULL, &buf_out);
    
    return is_idle;
}

BOOL is_pool_empty(VOID)
{
    BUFFER_OBJ buf_out;
    BOOL is_empty;
    
    ddf_DeviceIoControl(IOCTL_FPGA_GET_MEMORY_POOL_EMPTY, &fpga, NULL, &buf_out);
    
    return is_empty;
}

INT set_page_end_addr(UINT addr)
{
    BUFFER_OBJ buf_in;
    buf_in.pbuffer = &addr;
    
    return ddf_DeviceIoControl(IOCTL_FPGA_SET_MEMORY_POOL_END_ADDRESS, &fpga, &buf_in, NULL);
}

INT get_page_end_addr(VOID)
{
    BUFFER_OBJ buf_out;
    UINT addr;
    buf_out.pbuffer = &addr;
    
    ddf_DeviceIoControl(IOCTL_FPGA_GET_MEMORY_POOL_PAGE_END_ADDRESS, &fpga, NULL, &buf_out);
    
    return addr;
}

INT set_pool_end_addr(UINT addr)
{
    BUFFER_OBJ buf_in;
    buf_in.pbuffer = &addr;
    
    return ddf_DeviceIoControl(IOCTL_FPGA_SET_MEMORY_POOL_END_ADDRESS, &fpga, &buf_in, NULL);
}

INT get_pool_end_addr(VOID)
{
    BUFFER_OBJ buf_out;
    UINT addr;
    buf_out.pbuffer = &addr;
    
    ddf_DeviceIoControl(IOCTL_FPGA_GET_MEMORY_POOL_END_ADDRESS, &fpga, NULL, &buf_out);
    
    return addr;
}

INT light_leds(UINT leds)
{
    BUFFER_OBJ buf_in;
    buf_in.pbuffer = &leds;
    
    return ddf_DeviceIoControl(IOCTL_FPGA_SET_LED, &fpga, &buf_in, NULL);
}

INT get_pool_data(PUINT pbuf, UINT size)
{
    BUFFER_OBJ buf_out;
    buf_out.pbuffer = pbuf;
    buf_out.size = size;
    
    return ddf_DeviceIoControl(IOCTL_FPGA_GET_POOL_DATA, &fpga, NULL, &buf_out);
}

INT get_srcs_pending(VOID)
{
    BUFFER_OBJ buf_out;
    INT srcs_pending;
    
    buf_out.pbuffer = &srcs_pending;
    
    ddf_DeviceIoControl(IOCTL_FPGA_GET_SRCS_PENDING, &fpga, NULL, &buf_out);
    
    return srcs_pending;
}