//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//---------------------------------------------------------------------------
// applications for tests ,this file should be compiled by C++ compiler
//---------------------------------------------------------------------------
#include "../../../../inc/typedef.h"
#include "../../../../inc/driver.h"
#include "../../../../inc/gui.h"
#include "../../../../inc/task.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "./test.h"

void app_test_memory(void)
{
    DEVICE_OBJ leds;
    ddf_DeviceOpen(&leds, LEDs, 0);
    
    //should disable D and I caches before testing memory
    DEVICE_OBJ cp15;
    ddf_DeviceOpen(&cp15, CO_PROCESSOR15, 0);
    ddf_DeviceIoControl(IOCTL_ICACHE_DISABLE, &cp15, 0, 0);
    ddf_DeviceIoControl(IOCTL_DCACHE_CLEAN, &cp15, 0, 0);
    ddf_DeviceIoControl(IOCTL_DCACHE_DISABLE, &cp15, 0, 0);
    
  
    CHAR time[] = __TIME__;
    CHAR date[] = __DATE__;
    printf("Build %s, %s\n", time, date);
   
    //test memory
    UINT index;
    UINT i = 0;
    volatile PUINT mem = (PUINT)0x32000000;
    
    printf("\nTesting memory...\n\nProgress:");
    
    while(i++ < 10){
        for(index = 0; index < 0x400000; index++)
            mem[index] = index;
           
        for(index = 0; index < 0x400000; index++)
            if(mem[index] != index){
                printf("A memory error was detected at 0x%x .\n", 
                            (UINT)(mem+index));
                exit(1);
            }
            
        ddf_DeviceIoControl(
                IOCTL_LEDS_ON_OFF, 
                &leds, 
                (PBUFFER_OBJ)(i << 1), 
                0);
                
        printf("#");
    }
    
    printf("\n\nNo error was detected.\n\n");
    
    ddf_DeviceIoControl(IOCTL_ICACHE_FLUSH, &cp15, 0, 0);
    ddf_DeviceIoControl(IOCTL_ICACHE_ENABLE, &cp15, 0, 0);
    ddf_DeviceIoControl(IOCTL_DCACHE_FLUSH, &cp15, 0, 0);
    ddf_DeviceIoControl(IOCTL_DCACHE_ENABLE, &cp15, 0, 0);
    ddf_DeviceClose(&cp15, 0, 0);
    
    ddf_DeviceClose(&leds, 0, 0);
    
    while(1)
        ;
}

void app_test_lcd(void)
{
    UINT (*gmem)[X_SIZE];
    UINT X, Y;
    
    DEVICE_OBJ lcd;
    ddf_DeviceOpen(&lcd, LCD, 0);
    
    ddf_DeviceIoControl(
            IOCTL_LCD_BUFFER_MODE_SET, 
            &lcd, 
            (PBUFFER_OBJ)2, 
            0);
            
    ddf_DeviceIoControl(
            IOCTL_LCD_BUFFER_OPEN, 
            &lcd, 
            0, 
            (PBUFFER_OBJ)&gmem);
            
    ddf_DeviceIoControl(
            IOCTL_LCD_ON_OFF, 
            &lcd, 
            (PBUFFER_OBJ)1, 
            0);      
    
    while(1){
        for(Y = 0; Y < Y_SIZE; Y++)
            for(X = 0; X < X_SIZE; X++)
                gmem[Y][X] = RED;   
                 
        ddf_DeviceIoControl(
                IOCTL_LCD_BUFFER_TRIGGER, 
                &lcd, 
                0, 
                (PBUFFER_OBJ)&gmem);
        
        for(X = 0; X < 0x7fffff; X++)
            continue;   //delay
       
        for(Y = 0; Y < Y_SIZE; Y++)
            for(X = 0; X < X_SIZE; X++)
                gmem[Y][X] = BLUE;
                
        ddf_DeviceIoControl(
                IOCTL_LCD_BUFFER_TRIGGER, 
                &lcd, 
                0, 
                (PBUFFER_OBJ)&gmem);
        
        for(X = 0; X < 0x7fffff; X++)
            continue;   //delay
    }
    
    ddf_DeviceIoControl(
        IOCTL_LCD_BUFFER_CLOSE, 
        &lcd, 
        0, 
        0);
        
    ddf_DeviceClose(&lcd, 0, 0);
}

void app_test_color_display(void)
{
    struct{
        UINT color;
        CHAR name[20];
    }color_name[] = {
        BLUE, "",         
        GREEN, "",  
        RED, "", 
        CYAN, "",   
        MAGENTA, "",  
        YELLOW, "",
        LIGHTBLUE, "ǳ",
        LIGHTGREEN, "ǳ",
        LIGHTRED, "ǳ",
        LIGHTCYAN, "ǳ",
        LIGHTMAGENTA, "ǳ",
        LIGHTYELLOW, "ǳ",
        DARKBLUE, "",
        DARKGREEN, "",
        DARKRED, "",
        DARKCYAN, "",
        DARKMAGENTA, "",
        DARKYELLOW, "",
        WHITE, "",
        LIGHTGRAY, "ǳ", 
        GRAY, "",
		DARKGRAY, "",  
	    BLACK, "",
        BROWN, "" 
    };
    
    UINT x, y;
    UINT i;
    
    for(y = 0; y < Y_SIZE; y++)
        for(x = 0; x < X_SIZE; x++)
            _put_pixel(x, y, 0xd0d0ff);
    
    for(y = 4; y < Y_SIZE - 4; y++){
        for(x = 4; x < 124; x++)
            _put_pixel(x, y, WHITE);
        for(x = 128; x < X_SIZE - 4; x++)
            _put_pixel(x, y, YELLOW);
    }
            
    y = 0;
    
    for(i = 0; i < 24; i++){
        while(y++ < (i + 1) * 13)
            for(x = 0; x < 120; x++)
                _put_pixel(
                        x + 4, 
                        y + 3, 
                        color_name[i].color);
        
        _lcd_printf(
                x + 40, 
                y - 10, 
                _sys_text_class_get(), 
                color_name[i].name);
    }
    
    while(TRUE)
        continue;
}

void app_move_draw(void)
{
    
    UINT pos = 20;
    
    while(TRUE){
    
        for(UINT i = 0; i < 0x007fffff; i++)
            continue;
        
        for(UINT x = 20; x < 100; x++)
            for(UINT y = 20; y < 40; y++)
                _put_pixel(x, y, BLACK);
                

            for(UINT y = 20; y < 40; y++)
                for(UINT x = 20; x < pos; x++)
                    _put_pixel(x, y, BLUE);
        
        if(pos++ == 100)            
            pos = 20;
    }
}

void app_test_font_lib(void)
{
    UINT x, y;
    CHAR buf[LCD_PRINT_BUFFER_LENGTH_MAX];
    
    for(y = 0; y < Y_SIZE; y++)
        for(x = 0; x < X_SIZE; x++)
            _put_pixel(x, y, WHITE);
           
    y = 0;
    
    while(1){
        gets(buf);
        if(strlen(buf)){
            _lcd_printf(0, y, _sys_text_class_get(), buf);
            y += _sys_text_class_get()->pascii_font->char_height;
        }
    }
}

void app_timer_interrupt_test(void)
{
    DEVICE_OBJ ints_ctrl, timer;
    ddf_DeviceOpen(&ints_ctrl, INTs, NULL);
    ddf_DeviceOpen(&timer, TIMERs, NULL);
    
    TIMER_PARAM timer_param;
    BUFFER_OBJ buf_in;
    
    timer_param.id = 4;     //timer 4
    timer_param.param = 5;  //5ms
    
    buf_in.pbuffer = &timer_param;
    buf_in.size = sizeof(TIMER_PARAM);
    
    ddf_DeviceIoControl(
            IOCTL_TIMER_TIME_SET, 
            &timer, 
            &buf_in, 
            NULL);
    
    timer_param.param = ON;

    ddf_DeviceIoControl(
            IOCTL_TIMER_INT_ENABLE, 
            &ints_ctrl, 
            (PBUFFER_OBJ)4, 
            NULL);
            
    ddf_DeviceIoControl(
        IOCTL_TIMER_ON_OFF, 
        &timer, 
        &buf_in, 
        NULL);
    
    while(1)
    {
        printf("abcd\n");
    }
}

void app_draw_lib_test(void)
{

    UINT d = 1;
    UINT delay = 0x000fff;
    UINT t = 1;
    while(1){
        
        for(UINT x = 0; x < 240; x++)
            _line(x, 0, x, 319, x << 16 | GREEN | 255 - x);
        
        for(UINT x = 0; x < 240; x += d)
            _line(x, 0, 239 - x, 319, BLUE);
        
        for(UINT y = 0; y < 320; y += d)
            _line(0, y, 239, 319 - y, RED);
        
        _lcd_printf(
            0, 
            0, 
            _sys_text_class_get(), 
            "ֱ߻");
                              
        _draw_buffer_trigger();
        
        while(delay--)
            ;
        
        delay = 0x000fff;
        
        if(t)
            if(d++ > 50)
                t ^= 1; 
                
        if(!t)
            if(d-- < 3)
                t ^= 1; 
        
    }

/*
    RECT ret;
    ret.left = 100;
    ret.top = 100;
    ret.right = 200;
    ret.bottom = 200;
    _filled_rectangle(&ret, GREEN);
    
    _line(0, 0, 239, 319, RED);
    
    _draw_buffer_trigger();
    
    while(TRUE)
        continue;
        
*/
}

void app_multi_task_test(void)
{
    UINT task1(PVOID);
    UINT task2(PVOID);
    
    const UINT STACK_LENGTH = 1024 * 1;
   
   //allocate stack memory in user stack
    PUINT task1_stack = (PUINT)malloc(STACK_LENGTH);
    PUINT task2_stack = (PUINT)malloc(STACK_LENGTH);
    PUINT task3_stack = (PUINT)malloc(STACK_LENGTH);

    
    UID pid_task1, pid_task2, pid_task3;
 
    
    
/*
    pid_task1 = task_create(
                    (TASK_ROUTINE)app_draw_lib_test, 
                    0,                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  
                    task1_stack, 
                    STACK_LENGTH, 
                    0);
            
    pid_task2 = task_create(
                    (TASK_ROUTINE)app_test_color_display, 
                    0, 
                    task2_stack, 
                    STACK_LENGTH, 
                    0);
            
    pid_task3 = task_create(
                    (TASK_ROUTINE)app_move_draw,
                    0,
                    task3_stack,
                    STACK_LENGTH,
                    0);
    
    printf("pids for the three tasks are %d, %d, %d\n",
                        pid_task1, pid_task2, pid_task3);
*/                        
                        
                       
    task_create(
            task1,
            (PCHAR)"hello task1!\n",
            task1_stack,
            STACK_LENGTH,
            0);
            
    task_create(
            task2,
            (PCHAR)"",
            task2_stack,
            STACK_LENGTH,
            0);
    
    task_create(
            (TASK_ROUTINE)app_draw_lib_test,
            0,
            task3_stack,
            STACK_LENGTH,
            0);
                      
    os_start(); 
}

UINT task1(PVOID arg)
{
    while(TRUE)
        printf((PCHAR)arg);
    
    return 1;
}

UINT task2(PVOID arg)
{
    while(TRUE)
        printf((PCHAR)arg);
        
   
   PVOID a;
   PVOID b;
        
   memcpy(a, b, 10);
        
    return 1;
}

void app_test_fpga(void)
{
    PUINT pt = (PUINT)(0x8000000);
    pt[15] = 0x0;
    
    pt[3] = 0xf;
    
    pt[7] |= BIT9;
    
    while(!(pt[7] & BIT14)){
        printf("sample...\n");
    }
    
    UINT i = 0;
    while(!(pt[6] & BIT0)){
        printf("[0x%x] = %x\n", i, pt[5] & 0xffff);
        i++;
    }
}

