//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
// driver for fpga(Signal Acquisition And Communication Board)
//--------------------------------------------------------------------------

#include "../../../inc/typedef.h"
#include "../../../inc/ddf_api.h"
#include "./fpga.h"
#include <math.h>

BEGIN_IO_REQUEST_MAP(io_request_map)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_SAMPLE_START, fpga_set_sample_start)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_SAMPLE_STOP, fpga_set_sample_stop)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_TRIGGER_ON, fpga_set_trigger_on)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_TRIGGER_OFF, fpga_set_trigger_off)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_TRIGGER_SRC_CHA, fpga_set_trigger_src_cha)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_TRIGGER_SRC_CHB, fpga_set_trigger_src_chb)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_TRIGGER_RAISING, fpga_set_trigger_raising)              
    IOCTL_ROUTINE(IOCTL_FPGA_SET_TRIGGER_FALLING, fpga_set_trigger_falling)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_TRIGGER_ON_OFF, fpga_get_trigger_on_off)             
    IOCTL_ROUTINE(IOCTL_FPGA_GET_TRIGGER_RAISING_FALLING, fpga_get_trigger_raising_falling)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_TRIGGER_SRC_CHA_CHB, fpga_get_trigger_src_cha_chb)     
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_ADC_DFS_OFFSET, fpga_set_adc_dfs_offset)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_ADC_DFS_TWO_COMPLEMENT, fpga_set_adc_dfs_two_complement)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_ADC_DFS, fpga_get_adc_dfs)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_ADC_IDLE_BUSY, fpga_get_adc_idle_busy)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_SAMPLE_FREQ_AUTO, fpga_set_sample_freq_auto)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_SAMPLE_FREQ_MANUAL, fpga_set_sample_freq_manual)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_SAMPLE_FREQ, fpga_set_sample_freq)
    IOCTL_ROUTINE(IOCTL_FPGA_REQ_SIGNAL_FREQ_CALU, fpga_req_signal_freq_calu)  
    IOCTL_ROUTINE(IOCTL_FPGA_GET_SAMPLE_FREQ_AUTO_MANUAL, fpga_get_sample_freq_auto_manual)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_SIGNAL_FREQ_CALU_STATE, fpga_get_signal_freq_calu_state)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_MEMORY_POOL_PAGE_END_ADDRESS, fpga_set_memory_pool_page_end_address)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_MEMORY_POOL_PAGE_END_ADDRESS, fpga_get_memoty_pool_page_end_address)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_MEMORY_POOL_END_ADDRESS, fpga_set_memory_pool_end_address)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_MEMORY_POOL_END_ADDRESS, fpga_get_memory_pool_end_address)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_MEMORY_POOL_EMPTY, fpga_get_memory_pool_empty)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_LED, fpga_set_led)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_LED, fpga_get_led)   
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_DC_COUPLE, fpga_set_cha_dc_couple)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_AC_COUPLE, fpga_set_cha_ac_couple)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_PRE_ATTENUATOR_TO_X1, fpga_set_cha_pre_attenuator_to_x1)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_PRE_ATTENUATOR_TO_X10, fpga_set_cha_pre_attenuator_to_x10)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_PRE_ATTENUATOR_TO_X100, fpga_set_cha_pre_attenuator_to_x100)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_DC_COUPLE, fpga_set_chb_dc_couple)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_AC_COUPLE, fpga_set_chb_ac_couple)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_PRE_ATTENUATOR_TO_X1, fpga_set_chb_pre_attenuator_to_x1)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_PRE_ATTENUATOR_TO_X10, fpga_set_chb_pre_attenuator_to_x10)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_PRE_ATTENUATOR_TO_X100, fpga_set_chb_pre_attenuator_to_x100)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_SHIFT_VOLTAGE, fpga_set_cha_shift_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_TRIGGER_VOLTAGE, fpga_set_cha_trigger_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHA_VGA_GAIN, fpga_set_cha_vga_gain)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_SHIFT_VOLTAGE, fpga_set_chb_shift_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_TRIGGER_VOLTAGE, fpga_set_chb_trigger_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_SET_CHB_VGA_GAIN, fpga_set_chb_vga_gain)
    
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHA_COUPLE, fpga_get_cha_couple)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHA_PRE_ATTENUATOR, fpga_get_cha_pre_attenuator)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHB_COUPLE, fpga_get_chb_couple)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHB_PRE_ATTENUATOR, fpga_get_chb_pre_attenuator)
    
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHA_SHIFT_VOLTAGE, fpga_get_cha_shift_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHA_TRIGGER_VOLTAGE, fpga_get_cha_trigger_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHA_VGA_GAIN, fpga_get_cha_vga_gain)
    
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHB_SHIFT_VOLTAGE, fpga_get_chb_shift_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHB_TRIGGER_VOLTAGE, fpga_get_chb_trigger_voltage)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_CHB_VGA_GAIN, fpga_get_chb_vga_gain)
    
    IOCTL_ROUTINE(IOCTL_FPGA_GET_SINGAL_FREQ_CALU_COUNTER, fpga_get_signal_freq_calu_counter)
    
    //IOCTL_ROUTINE(IOCTL_FPGA_GET_SRAM_EMPTY, fpga_get_sram_empty)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_DMA_SERVICE, fpga_enable_dma_service)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_DMA_SERVICE, fpga_disable_dma_service)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_SRAM_FULL, fpga_enable_int_sram_full)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_SRAM_FULL, fpga_disable_int_sram_full)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_HC595_DONE, fpga_enable_int_hc595_done)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_HC595_DONE, fpga_disable_int_hc595_done)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_TLV5631_DONE, fpga_enable_int_tlv5631_done)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_TLV5631_DONE, fpga_disable_int_tlv5631_done)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_ARM_SELECTED, fpga_enable_int_arm_selected)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_ARM_SELECTED, fpga_disable_int_arm_selected)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_HOST_SELECTED, fpga_enable_int_host_selected)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_HOST_SELECTED, fpga_disable_int_host_selected)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_FREQ_DONE, fpga_enable_int_freq_done)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_FREQ_DONE, fpga_disable_int_freq_done)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_FREQ_OUT_OF_RANGE, fpga_enable_int_freq_out_of_range)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_FREQ_OUT_OF_RANGE, fpga_disable_int_freq_out_of_range)
    
    IOCTL_ROUTINE(IOCTL_FPGA_ENABLE_INT_WRITE_EP_DONE, fpga_enable_int_write_ep_done)
    IOCTL_ROUTINE(IOCTL_FPGA_DISABLE_INT_WRITE_EP_DONE, fpga_disable_int_write_ep_done)
    
    IOCTL_ROUTINE(IOCTL_FPGA_GET_SRCS_PENDING, fpga_get_srcs_pending)
    
    IOCTL_ROUTINE(IOCTL_FPGA_CLR_SRCS_SRAM_FULL, fpga_clr_srcs_sram_full)
    IOCTL_ROUTINE(IOCTL_FPGA_CLR_SRCS_HC595_DONE, fpga_clr_srcs_hc595_done)
    IOCTL_ROUTINE(IOCTL_FPGA_CLR_SRCS_TLV5631_DONE, fpga_clr_srcs_tlv5631_done)
    IOCTL_ROUTINE(IOCTL_FPGA_CLR_SRCS_FREQ_DONE, fpga_clr_srcs_freq_done)
    IOCTL_ROUTINE(IOCTL_FPGA_CLR_SRCS_FREQ_OUT_OF_RANGE, fpga_clr_srcs_freq_out_of_range)
    
    IOCTL_ROUTINE(IOCTL_FPGA_GET_POOL_DATA, fpga_get_pool_data)
    
    IOCTL_ROUTINE(IOCTL_FPGA_SET_HOST_ARM_SELECT, fpga_set_host_arm_select)
    IOCTL_ROUTINE(IOCTL_FPGA_GET_HOST_ARM_SELECT, fpga_get_host_arm_select)
END_IO_REQUEST_MAP()

static FPGA_REG * const regs = (PFPGA_REG)0x8000000;

static UINT V_cha_shift_revise = 4;
static UINT V_chb_shift_revise = 8;

static UINT V_adc_center = 990;
static UINT V_dac_ref = 1024;

static INT V_cha_shift = 2048;
static INT V_cha_trigger = 0;
static DOUBLE cha_vga_gain = 1;
static DOUBLE cha_pre_attenuator = 2;

static INT V_chb_shift = 2048;
static INT V_chb_trigger = 0;
static DOUBLE chb_vga_gain = 1;
static DOUBLE chb_pre_attenuator = 2;

INT fpga_init(void)
{
    UINT shift_volt, srcs_pending;
    BUFFER_OBJ buf_in, buf_out;
    
    shift_volt = 0;
    buf_in.pbuffer = &shift_volt;
    
    srcs_pending = 0;
    buf_out.pbuffer = &srcs_pending;
    
    fpga_set_adc_dfs_offset(NULL, NULL);
    
    fpga_set_cha_shift_voltage(&buf_in, NULL);
    
    do{
        fpga_get_srcs_pending(NULL, &buf_out);
    }while(!(srcs_pending & BIT2));
    
    fpga_clr_srcs_tlv5631_done(NULL, NULL);
    
    fpga_set_chb_shift_voltage(&buf_in, NULL);
    
    do{
        fpga_get_srcs_pending(NULL, &buf_out);
    }while(!(srcs_pending & BIT2));
    
    fpga_clr_srcs_tlv5631_done(NULL, NULL);
    
    return DDF_SUCCESS;
}

UID fpga_open(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT fpga_close(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT fpga_driver_entry(
            UINT            ioctl, 
    const   BUFFER_OBJ *   pbuffer_in, 
            BUFFER_OBJ *   pbuffer_out
    )
{
    return _ddf_driver_dispatch(
                io_request_map, 
                ioctl, 
                pbuffer_in, 
                pbuffer_out
                );
}

INT fpga_set_sample_start
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT9;
    return DDF_SUCCESS;
}

INT fpga_set_sample_stop
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT8;
    return DDF_SUCCESS;
}
 
INT fpga_set_trigger_on
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT12;
    return DDF_SUCCESS;
}
 
INT fpga_set_trigger_off
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL &= ~BIT12;
    return DDF_SUCCESS;
}

INT fpga_set_trigger_src_cha
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT11;
    return DDF_SUCCESS;
}

INT fpga_set_trigger_src_chb
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL &= ~BIT11;
    return DDF_SUCCESS;
}

INT fpga_set_trigger_raising
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT10;
    return DDF_SUCCESS;
}

INT fpga_set_trigger_falling
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL &= ~BIT10;
    return DDF_SUCCESS;
}

INT fpga_get_trigger_on_off
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SAMPLE_CTRL & BIT12) >> 12;
    return DDF_SUCCESS;
}

INT fpga_get_trigger_raising_falling
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SAMPLE_CTRL & BIT10) >> 10;
    return DDF_SUCCESS;
}

INT fpga_get_trigger_src_cha_chb
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SAMPLE_CTRL & BIT11) >> 11;
    return DDF_SUCCESS;
}     

INT fpga_set_adc_dfs_offset
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL &= ~BIT13;
    return DDF_SUCCESS;
}

INT fpga_set_adc_dfs_two_complement
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT13;
    return DDF_SUCCESS;
}

INT fpga_get_adc_dfs
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SAMPLE_CTRL & BIT13) >> 13;
    return DDF_SUCCESS;
}

INT fpga_get_adc_idle_busy
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SAMPLE_CTRL & BIT14) >> 14;
    return DDF_SUCCESS;
}

INT fpga_set_sample_freq_auto
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT7;
    return DDF_SUCCESS;
}

INT fpga_set_sample_freq_manual
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL &= ~BIT7;
    return DDF_SUCCESS;
}

INT fpga_set_sample_freq
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL &= ~0xf;
    regs->SAMPLE_CTRL |= (*((PUINT)(pbuffer_in->pbuffer))) & 0xf;
    return DDF_SUCCESS;
}

INT fpga_req_signal_freq_calu
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SAMPLE_CTRL |= BIT4;
    return DDF_SUCCESS;
}

INT fpga_get_sample_freq_auto_manual
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_in->pbuffer)) = (regs->SAMPLE_CTRL & BIT7) >> 7;
    return DDF_SUCCESS;
}

INT fpga_get_signal_freq_calu_state
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SAMPLE_CTRL & (BIT6 | BIT5)) >> 5;
    return DDF_SUCCESS;
}

INT fpga_set_memory_pool_page_end_address
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->PAGE_END_ADDR = *(PUINT)(pbuffer_in->pbuffer);
    return DDF_SUCCESS;
}

INT fpga_get_memoty_pool_page_end_address
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->PAGE_END_ADDR) & 0xffff;
    return DDF_SUCCESS;
}

INT fpga_set_memory_pool_end_address
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->POOL_END_ADDR = *((PUINT)(pbuffer_in->pbuffer));
    return DDF_SUCCESS;
}

INT fpga_get_memory_pool_end_address
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->POOL_END_ADDR) & 0xffff;
    return DDF_SUCCESS;
}

INT fpga_get_memory_pool_empty
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->POOL_EMPTY) & 0xffff;
    return DDF_SUCCESS;
}

INT fpga_set_led
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->LED = 0;
    regs->LED = (*((PUINT)(pbuffer_in->pbuffer))) & 0xf;
    return DDF_SUCCESS;
}

INT fpga_get_led
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->LED) & 0xffff;
    return DDF_SUCCESS;
}

INT fpga_set_cha_dc_couple
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->HC595 &= ~BIT2;
    return DDF_SUCCESS;
}

INT fpga_set_cha_ac_couple
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->HC595 |= BIT2;
    return DDF_SUCCESS;
}

INT fpga_set_cha_pre_attenuator_to_x1
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    UINT temp = regs->HC595;
    
    cha_pre_attenuator = 0;
    
    temp &= ~(BIT3 | BIT4);
    temp |= BIT4;
    regs->HC595 = temp;
    
    return DDF_SUCCESS;
}

INT fpga_set_cha_pre_attenuator_to_x10
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    UINT temp = regs->HC595;
    
    cha_pre_attenuator = 1;
    
    temp &= ~(BIT3 | BIT4);
    temp |= BIT3;
    regs->HC595 = temp;
    
    return DDF_SUCCESS;
}

INT fpga_set_cha_pre_attenuator_to_x100
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    cha_pre_attenuator = 2;
    
    regs->HC595 &= ~(BIT3 | BIT4);
    
    return DDF_SUCCESS;
}

INT fpga_set_chb_dc_couple
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->HC595 &= ~BIT5;
    return DDF_SUCCESS;
}

INT fpga_set_chb_ac_couple
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->HC595 |= BIT5;
    return DDF_SUCCESS;
}

INT fpga_set_chb_pre_attenuator_to_x1
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    UINT temp = regs->HC595;
    
    chb_pre_attenuator = 0;
    
    temp &= ~(BIT6 | BIT7);
    temp |= BIT7;
    regs->HC595 = temp;
    
    return DDF_SUCCESS;
}

INT fpga_set_chb_pre_attenuator_to_x10
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    UINT temp = regs->HC595;
    
    chb_pre_attenuator = 1;
    
    temp &= ~(BIT6 | BIT7);
    temp |= BIT6;
    regs->HC595 = temp;
    
    return DDF_SUCCESS;
}

INT fpga_set_chb_pre_attenuator_to_x100
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{    
    chb_pre_attenuator = 2;
    
    regs->HC595 &= ~(BIT6 | BIT7);
    
    return DDF_SUCCESS;
}

INT fpga_set_cha_shift_voltage
(
    const   BUFFER_OBJ * pbuffer_in,
            BUFFER_OBJ * pbuffer_out
)
{
    INT V_shift;
    UINT V_dac;
    UINT code;
    
    V_shift = *((PINT)(pbuffer_in->pbuffer));
    if(V_shift > 1024 || V_shift < -1024){
        return DDF_FAILURE;
    }else{
        V_cha_shift = V_shift;
        V_dac = V_dac_ref - (V_shift + V_adc_center) / 2;
        code = V_dac / 2 + V_cha_shift_revise;
        regs->TLV5631 = DAC_WORD(DAC_INDEX_FOR_CHA_SHIFT, code);
        return DDF_SUCCESS;
    }   
}

INT fpga_set_cha_trigger_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    INT V_dac = *((PINT)(pbuffer_in->pbuffer));
    UINT code = V_dac / 2;
    if(code > 1023){
        return DDF_FAILURE;
    }else{
        V_cha_trigger = V_dac;
        regs->TLV5631 = DAC_WORD(DAC_INDEX_FOR_CHA_TRIGGER, code);
        return DDF_SUCCESS;
    }
}

INT fpga_set_cha_vga_gain
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    DOUBLE A;
    DOUBLE dB;
    INT V_gain;
    UINT V_dac;
    UINT code;
    
    A = *(PDOUBLE)(pbuffer_in->pbuffer);
    
    if(A < 1 || A > 15.85){
        return DDF_FAILURE;
    }else{
        cha_vga_gain = A;
        dB = 20 * log10(A);
        V_gain = ((dB - 12.65) / 19.7) * 1000;
        V_dac = V_gain + 1024;
        code = V_dac / 2;
        regs->TLV5631 = DAC_WORD(DAC_INDEX_FOR_CHA_VGA, code);
        return DDF_SUCCESS;
    }
}

INT fpga_set_chb_shift_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    INT V_shift;
    UINT V_dac;
    UINT code;
    
    V_shift = *((PINT)(pbuffer_in->pbuffer));
    if(V_shift > 1024 || V_shift < -1024){
        return DDF_FAILURE;
    }else{
        V_chb_shift = V_shift;
        V_dac = V_dac_ref - (V_shift + V_adc_center) / 2;
        code = V_dac / 2 + V_chb_shift_revise;
        regs->TLV5631 = DAC_WORD(DAC_INDEX_FOR_CHB_SHIFT, code);
        return DDF_SUCCESS;
    }  
}

INT fpga_set_chb_trigger_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    INT V_dac = *((PINT)(pbuffer_in->pbuffer));
    UINT code = V_dac / 2;
    if(code > 1023){
        return DDF_FAILURE;
    }else{
        V_chb_trigger = V_dac;
        regs->TLV5631 = DAC_WORD(DAC_INDEX_FOR_CHB_TRIGGER, code);
        return DDF_SUCCESS;
    }
}

INT fpga_set_chb_vga_gain
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    DOUBLE A;
    DOUBLE dB;
    INT V_gain;
    UINT V_dac;
    UINT code;
    
    A = *(PDOUBLE)(pbuffer_in->pbuffer);
    
    if(A < 1 || A > 15.85){
        return DDF_FAILURE;
    }else{
        chb_vga_gain = A;
        dB = 20 * log10(A);
        V_gain = ((dB - 12.65) / 19.7) * 1000;
        V_dac = V_gain + 1024;
        code = V_dac / 2;
        regs->TLV5631 = DAC_WORD(DAC_INDEX_FOR_CHB_VGA, code);
        return DDF_SUCCESS;
    }
}

INT fpga_get_cha_couple
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->HC595 & BIT2) >> 2;
    return DDF_SUCCESS;
}

INT fpga_get_cha_pre_attenuator
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PDOUBLE)(pbuffer_out->pbuffer)) = cha_pre_attenuator;
    return DDF_SUCCESS;
}

INT fpga_get_chb_couple
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->HC595 & BIT5) >> 5;
    return DDF_SUCCESS;
}

INT fpga_get_chb_pre_attenuator
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PDOUBLE)(pbuffer_out->pbuffer)) = chb_pre_attenuator;
    return DDF_SUCCESS;
}

INT fpga_get_cha_shift_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PINT)(pbuffer_out->pbuffer)) = V_cha_shift;
    return DDF_SUCCESS;
}

INT fpga_get_cha_trigger_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PINT)(pbuffer_out->pbuffer)) = V_cha_trigger;
    return DDF_SUCCESS;
}

INT fpga_get_cha_vga_gain
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PDOUBLE)(pbuffer_out->pbuffer)) = cha_vga_gain;
    return DDF_SUCCESS;
}

INT fpga_get_chb_shift_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PINT)(pbuffer_out->pbuffer)) = V_chb_shift;
    return DDF_SUCCESS;
}

INT fpga_get_chb_trigger_voltage
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PINT)(pbuffer_out->pbuffer)) = V_chb_trigger;
    return DDF_SUCCESS;
}

INT fpga_get_chb_vga_gain
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PDOUBLE)(pbuffer_out->pbuffer)) = chb_vga_gain;
    return DDF_SUCCESS;
}

INT fpga_get_signal_freq_calu_counter
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->FREQ_CALU_CNT) & 0xffff;
    return DDF_SUCCESS;
}

INT fpga_get_sram_empty
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    return fpga_get_memory_pool_empty(pbuffer_in, pbuffer_out);
}

INT fpga_enable_dma_service
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT8;
    return DDF_SUCCESS;
}

INT fpga_disable_dma_service
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT8;
    return DDF_SUCCESS;
}

INT fpga_enable_int_sram_full
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT0;
    return DDF_SUCCESS;
}

INT fpga_disable_int_sram_full
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT0;
    return DDF_SUCCESS;
}

INT fpga_enable_int_hc595_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT1;
    return DDF_SUCCESS;
}

INT fpga_disable_int_hc595_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT1;
    return DDF_SUCCESS;
}

INT fpga_enable_int_tlv5631_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT2;
    return DDF_SUCCESS;
}

INT fpga_disable_int_tlv5631_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT2;
    return DDF_SUCCESS;
}

INT fpga_enable_int_arm_selected
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT3;
    return DDF_SUCCESS;
}

INT fpga_disable_int_arm_selected
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT3;
    return DDF_SUCCESS;
}

INT fpga_enable_int_host_selected
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT4;
    return DDF_SUCCESS;
}

INT fpga_disable_int_host_selected
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT4;
    return DDF_SUCCESS;
}

INT fpga_enable_int_freq_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT5;
    return DDF_SUCCESS;
}

INT fpga_disable_int_freq_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT5;
    return DDF_SUCCESS;
}

INT fpga_enable_int_freq_out_of_range
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT6;
    return DDF_SUCCESS;
}

INT fpga_disable_int_freq_out_of_range
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT6;
    return DDF_SUCCESS;
}

INT fpga_enable_int_write_ep_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK &= ~BIT7;
    return DDF_SUCCESS;
}

INT fpga_disable_int_write_ep_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->INTS_MASK |= BIT7;
    return DDF_SUCCESS;
}

INT fpga_get_srcs_pending
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    *((PUINT)(pbuffer_out->pbuffer)) = (regs->SRCS_PENDING) & 0xffff;
    return DDF_SUCCESS;
}

INT fpga_clr_srcs_sram_full
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SRCS_PENDING = BIT0;
    return DDF_SUCCESS;
}

INT fpga_clr_srcs_hc595_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SRCS_PENDING = BIT1;
    return DDF_SUCCESS;
}

INT fpga_clr_srcs_tlv5631_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SRCS_PENDING = BIT2;
    return DDF_SUCCESS;
}

INT fpga_clr_srcs_freq_done
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SRCS_PENDING = BIT5;
    return DDF_SUCCESS;
}

INT fpga_clr_srcs_freq_out_of_range
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    regs->SRCS_PENDING = BIT6;
    return DDF_SUCCESS;
}
    
INT fpga_get_pool_data
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    PUINT pbuffer = ((PUINT)(pbuffer_out->pbuffer));
    UINT buffer_size = pbuffer_out->size;
    UINT i;
    
    for(i = 0; i < buffer_size; i++)
        pbuffer[i] = (regs->POOL_DAT_PORT) & 0xffff;
        
    return DDF_SUCCESS;
}

INT fpga_set_host_arm_select
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    INT sel = *((PINT)(pbuffer_in->pbuffer));
    
    regs->HOST_ARM_SEL = sel;
    
    return DDF_SUCCESS;
}

INT fpga_get_host_arm_select
(
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
)
{
    (*(PUINT)(pbuffer_out->pbuffer)) = regs->HOST_ARM_SEL;
    return DDF_SUCCESS;
}