//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
//
//--------------------------------------------------------------------------

#ifndef _FPGA_H_
#define _FPGA_H_

#ifdef __cplusplus
extern"C"{
#endif

//should be located at 0x0800_0000(bank1)
typedef struct _fpga_reg{
    volatile UINT HC595;
    volatile UINT TLV5631;
    volatile UINT TLV5631_BAK;
    volatile UINT LED;
    volatile UINT POOL_END_ADDR;
    volatile UINT POOL_DAT_PORT;
    volatile UINT POOL_EMPTY;
    volatile UINT SAMPLE_CTRL;
    volatile UINT FREQ_CALU_CNT;
    volatile UINT PAGE_END_ADDR;
    volatile UINT SRCS_PENDING;
    volatile UINT INTS_MASK;
    volatile UINT INTS_PENDING;
    volatile UINT INTS_OFFSET;
    volatile UINT USB_CTRL;
    volatile UINT HOST_ARM_SEL;
}FPGA_REG, *PFPGA_REG;

#define     DAC_INDEX_FOR_CHA_SHIFT         0x6
#define     DAC_INDEX_FOR_CHA_TRIGGER       0x5
#define     DAC_INDEX_FOR_CHA_VGA           0x7
#define     DAC_INDEX_FOR_CHB_SHIFT         0x1
#define     DAC_INDEX_FOR_CHB_TRIGGER       0x2
#define     DAC_INDEX_FOR_CHB_VGA           0x0

#define     DAC_WORD(CH_INDEX, VALUE)   (((CH_INDEX) << 12) | ((VALUE) << 2))

INT fpga_init(void);
UID fpga_open(UINT, UINT);
INT fpga_close(UINT, UINT);
INT fpga_driver_entry(UINT, const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_set_sample_start(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_sample_stop(const BUFFER_OBJ *, BUFFER_OBJ *);
 
INT fpga_set_trigger_on(const BUFFER_OBJ *, BUFFER_OBJ *);               
INT fpga_set_trigger_off(const BUFFER_OBJ *, BUFFER_OBJ *);       
INT fpga_set_trigger_src_cha(const BUFFER_OBJ *, BUFFER_OBJ *);           
INT fpga_set_trigger_src_chb(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_trigger_raising(const BUFFER_OBJ *, BUFFER_OBJ *);              
INT fpga_set_trigger_falling(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_trigger_on_off(const BUFFER_OBJ *, BUFFER_OBJ *);               
INT fpga_get_trigger_raising_falling(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_trigger_src_cha_chb(const BUFFER_OBJ *, BUFFER_OBJ *);         

INT fpga_set_adc_dfs_offset(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_adc_dfs_two_complement(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_adc_dfs(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_adc_idle_busy(const BUFFER_OBJ *, BUFFER_OBJ *);   

INT fpga_set_sample_freq_auto(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_sample_freq_manual(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_sample_freq(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_req_signal_freq_calu(const BUFFER_OBJ *, BUFFER_OBJ *);            
INT fpga_get_sample_freq_auto_manual(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_signal_freq_calu_state(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_set_memory_pool_page_end_address(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_memoty_pool_page_end_address(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_memory_pool_end_address(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_memory_pool_end_address(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_memory_pool_empty(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_set_led(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_led(const BUFFER_OBJ *, BUFFER_OBJ *);          

INT fpga_set_cha_dc_couple(const BUFFER_OBJ *, BUFFER_OBJ *);           
INT fpga_set_cha_ac_couple(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_cha_pre_attenuator_to_x1(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_cha_pre_attenuator_to_x10(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_cha_pre_attenuator_to_x100(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_dc_couple(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_ac_couple(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_pre_attenuator_to_x1(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_pre_attenuator_to_x10(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_pre_attenuator_to_x100(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_set_cha_shift_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_cha_trigger_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_cha_vga_gain(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_set_chb_shift_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_trigger_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_set_chb_vga_gain(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_get_cha_couple(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_cha_pre_attenuator(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_chb_couple(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_chb_pre_attenuator(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_get_cha_shift_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_cha_trigger_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_cha_vga_gain(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_get_chb_shift_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_chb_trigger_voltage(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_chb_vga_gain(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_get_signal_freq_calu_counter(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_sram_empty(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_dma_service(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_dma_service(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_sram_full(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_sram_full(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_hc595_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_hc595_done(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_tlv5631_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_tlv5631_done(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_arm_selected(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_arm_selected(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_host_selected(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_host_selected(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_freq_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_freq_done(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_freq_out_of_range(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_freq_out_of_range(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_enable_int_write_ep_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_disable_int_write_ep_done(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_get_srcs_pending(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_clr_srcs_sram_full(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_clr_srcs_hc595_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_clr_srcs_tlv5631_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_clr_srcs_freq_done(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_clr_srcs_freq_out_of_range(const BUFFER_OBJ *, BUFFER_OBJ *);
    
INT fpga_get_pool_data(const BUFFER_OBJ *, BUFFER_OBJ *);

INT fpga_set_host_arm_select(const BUFFER_OBJ *, BUFFER_OBJ *);
INT fpga_get_host_arm_select(const BUFFER_OBJ *, BUFFER_OBJ *);
#ifdef __cplusplus
}
#endif

#endif