//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
// driver for lcd controller
//--------------------------------------------------------------------------
#include "../../../inc/typedef.h"
#include "../../../inc/ddf_api.h"
#include "./lcd.h"

BEGIN_IO_REQUEST_MAP(io_request_map)
    IOCTL_ROUTINE(IOCTL_LCD_BUFFER_TRIGGER, lcd_buffer_trigger)
    IOCTL_ROUTINE(IOCTL_LCD_GMEM_GET, lcd_gmem_get)
    IOCTL_ROUTINE(IOCTL_LCD_BUFFER_OPEN, lcd_buffer_open)  
    IOCTL_ROUTINE(IOCTL_LCD_BUFFER_CLOSE, lcd_buffer_close)
    IOCTL_ROUTINE(IOCTL_LCD_BUFFER_MODE_SET, lcd_buffer_mode_set)
    IOCTL_ROUTINE(IOCTL_LCD_ON_OFF, lcd_on_off)
END_IO_REQUEST_MAP()

//locate lcd controller special registers' start address
static LCD_REG * const regs = (PLCD_REG)0x4d000000;

//define horizontal size and vertical size of lcd screen 
static const UINT X_SIZE = 240;
static const UINT Y_SIZE = 320;

//define graphic memory, double buffers
static UINT gmem0[Y_SIZE][X_SIZE];
static UINT gmem1[Y_SIZE][X_SIZE];

//mark for which graphic memory being used by lcd controller
// 0 -- gmem0 is in use
// 1 -- gmem1 is in use
static UINT in_use_gmem = 0;

//buffer mode
// 1 -- single buffer mode
// 2 -- double buffer mode
static UINT buffer_mode = 1;

//parameters for LCDSADDR1, will be changed at runtime
static UINT LCDBANK;
static UINT LCDBASEU; 
    
//parameters for LCDSADDR2, will be changed at runtime
static UINT LCDBASEL;

INT lcd_init(void)
{ 
//Vertical size = 320, Horizontal size = 240, 24bpp TFT   
//Look datasheet of S3C2440 and datasheet of NL2432HC32 for more details
    
    //parameters for LCDCON1
    const UINT CLKVAL = /*(HCLK / VCLK / 2 - 1)*/ 3 << 8;
    const UINT MMODE = 0x00 << 7;
    const UINT PNRMODE = 0x03 << 5;
    const UINT BPPMODE = 0x0d << 1;
    const UINT ENVID = 0x00 << 0;
    
    //parameters for LCDCON2
    const UINT VBPD = 3 << 24;
    const UINT LINEVAL = (Y_SIZE - 1) << 14;
    const UINT VFPD = 3 << 6;
    const UINT VSPW = 2 << 0;
    
    //parameters for LCDCON3
    const UINT HBPD = 41 << 19;
    const UINT HOZVAL = (X_SIZE - 1) << 8;
    const UINT HFPD = 9 << 0;
    
    //parameters for LCDCON4;
    const UINT MVAL = 0x00 << 8;
    const UINT HSPW = 2 << 7;
    
    //parameters for LCDCON5;
    const UINT BPP24BL = 0x00 << 12;
    const UINT FRM565 = 0x00 << 11;
    const UINT INVVCLK = 0x01 << 10;
    const UINT INVVLINE = 0x01 << 9;
    const UINT INVVFRAME = 0x01 << 8;
    const UINT INVVD = 0x00 << 7;
    const UINT INVVDEN = 0x00 << 6;
    const UINT INVPWREN = 0x00 << 5;
    const UINT INVLEND = 0x00 << 4;
    const UINT PWREN = 0x01 << 3;
    const UINT ENLEND = 0x00 << 2;
    const UINT BSWP = 0x00 << 1;
    const UINT HWSWP = 0x00 << 0;
                               
    //parameters for LCDSADDR3
    const UINT OFFSIZE = 0x00 << 11;
    const UINT PAGEWIDTH = X_SIZE * 2 << 0;
    
    //parameters for REDLUT
    const UINT REDVAL = 0x00 << 0;
    
    //parameters for GREENLUT
    const UINT GREENVAL = 0x00 << 0;
    
    //parameters for BLUELUT
    const UINT BLUEVAL = 0x00 << 0;
    
    //parameters for TPAL
    const UINT TPALEN = 0x00 << 24;
    const UINT TPALVAL = 0x00 << 0;
    
    //parameters for LCDINTPND
    const UINT intpndINT_FrSyn = 0x00 << 1;
    const UINT intpndINT_FiCnt = 0x00 << 0;
    
    //parameters for LCDSRCPND
    const UINT srcpndINT_FrSyn = 0x00 << 1;
    const UINT srcpndINT_FiCnt = 0x00 << 0;
    
    //parameters for LCDINTMSK
    const UINT FIWSEL = 0x00 << 2;
    const UINT intmskINT_FrSyn = 0x01 << 1;
    const UINT intmskINT_FiCnt = 0x01 << 0;
    
    //parameters for TCONSEL
    const UINT LCC_SEL5 = 0x00 << 9;
    const UINT LCC_SEL4 = 0x00 << 8;
    const UINT LCC_SEL3 = 0x00 << 7;
    const UINT LCC_SEL2 = 0x00 << 6;
    const UINT LCC_SEL1 = 0x00 << 5;
    const UINT LCC_EN = 0x00 << 4;
    const UINT CPV_SEL = 0x00 << 3;
    const UINT MODE_SEL = 0x00 << 2;
    const UINT RES_SEL = 0x00 << 1;
    const UINT LPC_EN = 0x00 << 0;
    
//Step 1, configure GPIOs    
    GPIO_REG * const gpio = (PGPIO_REG)0x56000000; 
        
    //set GPIO C,D to lcd ctrl, datas and disable pullup 
    gpio->GPCCON = BIT31 | BIT29 | BIT27 | BIT25 | BIT23 
                   | BIT21| BIT19 | BIT17 | BIT15 | BIT13 
                   | BIT11 | BIT9 | BIT7 | BIT5 | BIT3 | BIT1;                                                
    gpio->GPDCON = BIT31 | BIT29 | BIT27 | BIT25 | BIT23
                   | BIT21 | BIT19 | BIT17 | BIT15 | BIT13 
                   | BIT11 | BIT9 | BIT7 | BIT5 | BIT3 | BIT1;
    gpio->GPCUP = 0xffffffff;
    gpio->GPDUP = 0xffffffff;
    
    //set GPG4 to LCD_PWR and pullup disable
    gpio->GPGCON &= ~(BIT8 | BIT9);
    gpio->GPGCON |= (BIT8 | BIT9);
    gpio->GPGUP |= BIT4;

//Step 2, configure lcd controller
    regs->LCDCON1 = CLKVAL | MMODE | PNRMODE | BPPMODE | ENVID;
    regs->LCDCON2 = VBPD | LINEVAL | VFPD | VSPW;
    regs->LCDCON3 = HBPD | HOZVAL | HFPD;
    regs->LCDCON4 = MVAL | HSPW;
    regs->LCDCON5 = BPP24BL | FRM565 | INVVCLK | INVVLINE
                    | INVVFRAME | INVVD | INVVDEN | INVPWREN
                    | INVLEND | PWREN | ENLEND |BSWP | HWSWP;
                    
    LCDBANK = ((UINT)gmem0 >> 22) << 21;
    LCDBASEU = ((UINT)gmem0 >> 1) & 0x1fffff;
    LCDBASEL = ((UINT)(*(gmem0 + Y_SIZE - 1) + X_SIZE) >> 1) & 0x1fffff;
    
    regs->LCDSADDR1 = LCDBANK | LCDBASEU;
    regs->LCDSADDR2 = LCDBASEL;
    regs->LCDSADDR3 = OFFSIZE | PAGEWIDTH;
    
    regs->REDLUT = REDVAL;
    regs->GREENLUT = GREENVAL;
    regs->BLUELUT = BLUEVAL;
    
    regs->TPAL = TPALEN | TPALVAL;
    
    regs->LCDINTPND = intpndINT_FrSyn | intpndINT_FiCnt;
    regs->LCDSRCPND = srcpndINT_FrSyn | srcpndINT_FiCnt;
    regs->LCDINTMSK = FIWSEL | intmskINT_FrSyn | intmskINT_FiCnt;
    
    regs->TCONSEL = LCC_SEL5 | LCC_SEL4 | LCC_SEL3 | LCC_SEL2
                    | LCC_SEL1 | LCC_EN | CPV_SEL | CPV_SEL
                    | MODE_SEL | RES_SEL | LPC_EN;                    

    return DDF_SUCCESS;
}

UID lcd_open(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT lcd_close(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT lcd_driver_entry(
            UINT            ioctl, 
    const   BUFFER_OBJ *    pbuffer_in, 
            BUFFER_OBJ *    pbuffer_out
    )
{
    return _ddf_driver_dispatch(
                io_request_map, 
                ioctl, 
                pbuffer_in, 
                pbuffer_out
                );
}

INT lcd_buffer_mode_set(
    const   BUFFER_OBJ *    pbuffer_in, 
            BUFFER_OBJ *    pbuffer_out
    )
{
    buffer_mode = (UINT)pbuffer_in;
    return DDF_SUCCESS;
}

INT lcd_buffer_open(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    if(buffer_mode == 1)
        *(PUINT)pbuffer_out = (UINT)(in_use_gmem == 0 ? gmem0 : gmem1);
    else if(buffer_mode == 2)
        *(PUINT)pbuffer_out = (UINT)(in_use_gmem == 0 ? gmem1 : gmem0);
    else
        return DDF_PARAMETER_ERROR;
        
    return DDF_SUCCESS;
}

INT lcd_buffer_trigger(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    UINT LINECNT;
    
    if(buffer_mode == 2){
        if(in_use_gmem == 0){
            LCDBANK = ((UINT)gmem1 >> 22) << 21;
            LCDBASEU = ((UINT)gmem1 >> 1) & 0x1fffff;
            LCDBASEL = ((UINT)(*(gmem1 + Y_SIZE - 1) + X_SIZE) >> 1) & 0x1fffff;
            in_use_gmem = 1;
            *(PUINT)pbuffer_out = (UINT)gmem0;
        }else{
            LCDBANK = ((UINT)gmem0 >> 22) << 21;
            LCDBASEU = ((UINT)gmem0 >> 1) & 0x1fffff;
            LCDBASEL = ((UINT)(*(gmem0 + Y_SIZE - 1) + X_SIZE) >> 1) & 0x1fffff;
            in_use_gmem = 0;
            *(PUINT)pbuffer_out = (UINT)gmem1;
        }
        
        while(1){
            LINECNT = (regs->LCDCON1 >> 8) & 0x3ff;
            if(LINECNT > 0){
                regs->LCDSADDR1 = LCDBANK | LCDBASEU;
                regs->LCDSADDR2 = LCDBASEL;
                break;
            }
        }
    }
    
    return DDF_SUCCESS;
}

INT lcd_buffer_close(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    UINT LINECNT;
    
    if(buffer_mode == 2){
        if(in_use_gmem == 0){
            LCDBANK = ((UINT)gmem1 >> 22) << 21;
            LCDBASEU = ((UINT)gmem1 >> 1) & 0x1fffff;
            LCDBASEL = ((UINT)(*(gmem1 + Y_SIZE - 1) + X_SIZE) >> 1) & 0x1fffff;
            in_use_gmem = 1;  
        }else{
            LCDBANK = ((UINT)gmem0 >> 22) << 21;
            LCDBASEU = ((UINT)gmem0 >> 1) & 0x1fffff;
            LCDBASEL = ((UINT)(*(gmem0 + Y_SIZE - 1) + X_SIZE) >> 1) & 0x1fffff;
            in_use_gmem = 0;
        }
        
        while(1){
            LINECNT = (regs->LCDCON1 >> 8) & 0x3ff;
            if(LINECNT > 0){
                regs->LCDSADDR1 = LCDBANK | LCDBASEU;
                regs->LCDSADDR2 = LCDBASEL;
                break;
            }
        }
        
    }
    
    return DDF_SUCCESS;
}

INT lcd_on_off(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    if((UINT)pbuffer_in == 0)
        regs->LCDCON1 &= ~0x01;
    else if((UINT)pbuffer_in == 1)
        regs->LCDCON1 |= 0x01;
    else
        return DDF_PARAMETER_ERROR;
        
    return DDF_SUCCESS;
}

INT lcd_gmem_get(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    *(PUINT)pbuffer_out = (UINT)(in_use_gmem == 0 ? gmem0 : gmem1);
    return DDF_SUCCESS;
}
