//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
// driver for leds
//--------------------------------------------------------------------------
#include "../../../inc/typedef.h"
#include "../../../inc/ddf_api.h"
#include "./leds.h"

BEGIN_IO_REQUEST_MAP(io_request_map)
    IOCTL_ROUTINE(IOCTL_LEDS_ALL_ON, leds_all_on)
    IOCTL_ROUTINE(IOCTL_LEDS_ALL_OFF, leds_all_off)
    IOCTL_ROUTINE(IOCTL_LEDS_ON_OFF, leds_on_off)
END_IO_REQUEST_MAP()

static GPIO_REG * const regs = (PGPIO_REG)0x56000000;

INT leds_init()
{
    //GPB 5,6,7,8 is led 1,2,3,4, low level ON
    
    //set GPB 5,6,7,8 to output and pullup disable
    regs->GPBCON &= ~(BIT10 | BIT11 | BIT12 | BIT13
                     | BIT14 | BIT15 | BIT16 | BIT17);
    regs->GPBCON |= BIT10 | BIT12 | BIT14 | BIT16;
    regs->GPBUP |= BIT5 | BIT6 | BIT7 | BIT8;
    
    //all the leds are OFF
    regs->GPBDAT |= BIT5 | BIT6 | BIT7 | BIT8;
    
    return DDF_SUCCESS;
}

UID leds_open(
        UINT    major,
        UINT    minor
        )
{
    return DDF_SUCCESS;
}

INT leds_close(
        UINT    major, 
        UINT    minor
        )
{
    return DDF_SUCCESS;
}

INT leds_driver_entry(
                UINT            ioctl, 
        const   BUFFER_OBJ *    pbuffer_in, 
                BUFFER_OBJ *    pbuffer_out
        )
{
    return _ddf_driver_dispatch(
                io_request_map, 
                ioctl, 
                pbuffer_in, 
                pbuffer_out
                );
}

INT leds_all_on(
        const   BUFFER_OBJ *    pbuffer_in, 
                BUFFER_OBJ *    pbuffer_out
        )
{
    regs->GPBDAT &= ~(BIT5|BIT6|BIT7|BIT8);
    return DDF_SUCCESS;
}

INT leds_all_off(
        const   BUFFER_OBJ *    pbuffer_in,
                BUFFER_OBJ *    pbuffer_out
        )
{
    regs->GPBDAT |= (BIT5|BIT6|BIT7|BIT8);
    return DDF_SUCCESS;
}

INT leds_on_off(
        const   BUFFER_OBJ *    pbuffer_in,
                BUFFER_OBJ *    pbuffer_out
        )
{
    UINT on = (UINT)pbuffer_in;
    regs->GPBDAT |= BIT5 | BIT6 | BIT7 | BIT8;
    regs->GPBDAT &= ~(on << 4);
    return DDF_SUCCESS;
}
