//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
// driver for timer
//--------------------------------------------------------------------------

#include "../../../inc/typedef.h"
#include "../../../inc/ddf_api.h"
#include "./device_type.h"
#include "./timers.h"

BEGIN_IO_REQUEST_MAP(io_request_map)
    IOCTL_ROUTINE(IOCTL_TIMER_TIME_SET, timer_time_set)
    IOCTL_ROUTINE(IOCTL_TIMER_ON_OFF, timer_on_off)
END_IO_REQUEST_MAP()

static TIMER_REG * const regs = (PTIMER_REG)0x51000000;
extern UINT PCLK;

INT timer_init(void)
{
    regs->TCNTB0 = 0;
    regs->TCMPB0 = 0;
    regs->TCNTO0 = 0;
    regs->TCNTB1 = 0;
    regs->TCMPB1 = 0;
    regs->TCNTO1 = 0;
    regs->TCNTB2 = 0;
    regs->TCMPB2 = 0;
    regs->TCNTO2 = 0;
    regs->TCNTB3 = 0;
    regs->TCMPB3 = 0;
    regs->TCNTO3 = 0;
    regs->TCNTB4 = 0;
    regs->TCNTO4 = 0;
    
    //set perscaler 1, 2 to 2MHz
    regs->TCFG0 = 0;
    regs->TCFG0 = PCLK / 2000000;  
    regs->TCFG0 |= PCLK / 2000000 << 8;
 
    //set MUX 4, 3, 2, 1 to 1/2 and set all 
    //timers are running under the interrupt mode
    regs->TCFG1 = 0;
    
    //now all timers will down count at every 1us
    
    return DDF_SUCCESS;
}

UID timer_open(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT timer_close(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT timer_driver_entry(
            UINT            ioctl, 
    const   BUFFER_OBJ *   pbuffer_in, 
            BUFFER_OBJ *   pbuffer_out
    )
{
    return _ddf_driver_dispatch(
                io_request_map, 
                ioctl, 
                pbuffer_in, 
                pbuffer_out
                );
}

INT timer_time_set(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    const TIMER_PARAM * ptimer_param 
            = (PTIMER_PARAM)(pbuffer_in->pbuffer);
            
    if(ptimer_param->param <= 65){     
        switch(ptimer_param->id)
        {
        case 4:
            regs->TCNTB4 = ptimer_param->param * 1000;
            regs->TCON = BIT22 | BIT21;
            return DDF_SUCCESS;
        default:
            return DDF_FAILURE;
        }
    }else{
        return DDF_PARAMETER_ERROR;
    }
}

INT timer_on_off(
    const   BUFFER_OBJ *    pbuffer_in,
            BUFFER_OBJ *    pbuffer_out
    )
{
    const TIMER_PARAM * ptimer_param 
            = (PTIMER_PARAM)(pbuffer_in->pbuffer);
            
    switch(ptimer_param->id)
    {
    case 4:
        switch(ptimer_param->param)
        {
        case ON:
            regs->TCON |= BIT20;
            regs->TCON &= ~BIT21;
            return DDF_SUCCESS;
        case OFF:
            regs->TCON &= ~BIT20;
            return DDF_SUCCESS;
        default:
            return DDF_PARAMETER_ERROR;
        }
    default:
        return DDF_FAILURE;
    }
}
