//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
// driver for uart
//--------------------------------------------------------------------------

#include "../../../inc/typedef.h"
#include "../../../inc/ddf_api.h"
#include "./uart.h"

BEGIN_IO_REQUEST_MAP(io_request_map)
    IOCTL_ROUTINE(IOCTL_UART_SEND_CHAR, uart_send_char)
    IOCTL_ROUTINE(IOCTL_UART_GET_CHAR, uart_get_char)
END_IO_REQUEST_MAP()

static UART_REG * const regs = (PUART_REG)0x50000000;

INT uart_init(void)
{
    extern UINT PCLK;
    extern UINT BAUD_RATE;
    
    //set GPH0,1,2,3,4 to nCTS0, nRTS0, nTXD0 and nRXD0 and pullup disable
    GPIO_REG * const gpio = (PGPIO_REG)0x56000000;
    gpio->GPHCON &= ~(BIT0 | BIT1 | BIT2 | BIT3
                     | BIT4 | BIT5 | BIT6 | BIT7);
    gpio->GPHCON |= BIT1 | BIT3 | BIT5 | BIT7;
    gpio->GPHUP |= BIT0 | BIT1 | BIT2 | BIT3 | BIT4;
    
    //one stop bit and 8bits word per frame, no parity
    regs->ULCON = BIT0 | BIT1;
    
    //transmit mode : interrupt request or polling mode
    //receive mode  : interrupt request or polling mode
    regs->UCON = BIT0 | BIT2;
    
    //disable fifo
    regs->UFCON = 0;
    
    //disable auto flow control
    regs->UMCON = 0;
    
    //set uart baud rate
    regs->UBRDIV = (UINT)(PCLK / (BAUD_RATE * 16)) - 1;
    
    return DDF_SUCCESS;
}

INT uart_open(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT uart_close(UINT major, UINT minor)
{
    return DDF_SUCCESS;
}

INT uart_driver_entry(
            UINT ioctl, 
    const   BUFFER_OBJ * pbuffer_in, 
            BUFFER_OBJ * pbuffer_out
    )
{
    return _ddf_driver_dispatch(
                io_request_map, 
                ioctl, 
                pbuffer_in, 
                pbuffer_out
                );
}     

INT uart_get_char(
    const   BUFFER_OBJ * pbuffer_in,
            BUFFER_OBJ * pbuffer_out
    )
{
    while(!(regs->UTRSTAT & 0x01))
        continue;
        
    *(PUINT)pbuffer_out = regs->URXH & 0xff;
    
    return DDF_SUCCESS;
}


INT uart_send_char(
    const   BUFFER_OBJ * pbuffer_in,
            BUFFER_OBJ * pbuffer_out
    )
{
    while(!(regs->UTRSTAT & 0x04))
        continue;
        
    if((UINT)pbuffer_in == '\n')
        regs->UTXH = '\r';
    else
        regs->UTXH = (UINT)pbuffer_in;
    
    return DDF_SUCCESS;
}

