//--------------------------------------------------------------------------
//Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
//--------------------------------------------------------------------------
//This file is part of ARMV4T_and_FPGA_based_DSO
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------

#ifndef _CP15_H_
#define _CP15_H_

#ifdef __cplusplus
extern"C"{
#endif

void    _cp15_mmu_enable(void);
void    _cp15_mmu_disable(void);
void    _cp15_mmu_tlb_flush(void);
void    _cp15_mmu_ttb_set(const UINT * ttb);
void    _cp15_mmu_domain_set(UINT val);
void    _cp15_mmu_process_id_set(UINT pid);

void    _cp15_DCache_flush(void);
void    _cp15_DCache_clean(void);
void    _cp15_DCache_enable(void);
void    _cp15_DCache_disable(void);
void    _cp15_ICache_flush(void);
void    _cp15_ICache_enable(void);
void    _cp15_ICache_disable(void);

void    _cp15_SetAsyncBusMode(void);
void    _cp15_SetFastBusMode(void);

// -- Structure --
// "region" alias "REGION" pointer "PREGION"
// -- Description --
// System memory region abstract
// -- Members --
// ...
typedef struct region{
    UINT vAddress;  //region start virtual address
    UINT page_num;  //region page num 
    UINT pAddress;  //region start physical address
    UINT AP;        //region access permission
    UINT CB;        //region cache attribute
    UINT DOM;       //region domain
}REGION, *PREGION;  

//Region cache and write buffer attributes
#define PTE_CB  0x3     //cache enable and write buffer enable
#define PTE_cB  0x1     //cache disable and write buffer enable
#define PTE_Cb  0x2     //cache enable and write buffer disable
#define PTE_cb  0x0     //cache disable and write buffer disable

//Domain values
#define NO_ACCESS   0x0     //Any access generates a permission fault
#define CLIENT      0x1     //Accesses are checked against the access permission bits in the section or page descripor
#define RESERVED    0x2     //Reserved, Currently behaves like the no access mode
#define MANAGER     0x3     //Accesses are not cheched against the access permission bits so a permission fault cannot be generated

//Region APs
#define RWRW        0x3     //Access allowed both in Supervisor mode and USER mode
#define RWRO        0x2     //Writes in User mode cause permission fault
#define RWNA        0x1     //Access allowed only in Supervisor mode

// ***init_page_table***
// -- Description --
// initialize all PTEs to FAULT
// -- Parameters --
// "pt", page table base address
// -- Return --
// None
void _page_table_init(UINT *pt);

void _region_init(const REGION * pRegion);
void _mmu_init(void);

#define PT_BASE     0x33ffc000    //L1 page table base address

#define BANK0       0x00000000
#define BANK1       0x08000000
#define BANK2       0x10000000
#define BANK3       0x18000000
#define BANK4       0x20000000  
#define BANK5       0x28000000 
#define BANK6       0x30000000
#define BANK7       0x38000000     
#define PERIPHERAL  0x40000000

#ifdef __cplusplus
}
#endif

#endif
