;--------------------------------------------------------------------------
;Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
;--------------------------------------------------------------------------
;This file is part of ARMV4T_and_FPGA_based_DSO

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either
;version 2 of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; -- This file contains some variables, addresses, macros ... 
; -- whatever, just mess.
;--------------------------------------------------------------------------

; -- Processor modes
ABORT_MODE          EQU             0x17
FIQ_MODE            EQU             0x11
IRQ_MODE            EQU             0x12
SVC_MODE            EQU             0x13
SYS_MODE            EQU             0x1f
UNDEF_MODE          EQU             0x1b
USER_MODE           EQU             0x10

MODE_MASK           EQU             0x1f
NOINT               EQU             (0x03<<6)
F_BIT               EQU             (0x01<<6)
I_BIT               EQU             (0x01<<7)

; -- Bit definitions about register c0 to control cp15
iA                  EQU             (0x01<<31)  ;Asynchronous clock select
nF                  EQU             (0x01<<30)  ;notFastBus select
RR                  EQU             (0x01<<14)  ;Round robin replacement
V                   EQU             (0x01<<13)  ;Base location of exception regiters
I                   EQU             (0x01<<12)  ;ICache enable
R                   EQU             (0x01<<9)   ;ROM protection
S                   EQU             (0x01<<8)   ;System protection
B                   EQU             (0x01<<7)   ;Endianness
C                   EQU             (0x01<<2)   ;DCache enable
A                   EQU             (0x01<<1)   ;Alignment fault enable
M                   EQU             (0x01<<0)   ;MMU enable

; -- Clock configuration
; -- The crystal @ 12MHz
; -- Set FCLK = 405MHz
M_MDIV              EQU             (0x7f<<12)
M_PDIV              EQU             (0x02<<4)
M_SDIV              EQU             (0x01<<0)    
; -- Set UCLK = 48MHz
U_MDIV              EQU             (0x38<<12)    
U_PDIV              EQU             (0x02<<4)
U_SDIV              EQU             (0x02<<0)
; -- Set FCLK : HCLK : PCLK = 1 : 4 : 8
CLKDIVN_VAL         EQU             0x05
    
; -- Memory arrangement
; -- Size of SDRAM = 64MB
VECTORS_JUMP_TABLE_BASE EQU         0x40000000  ;be located at internal SRAM
SDRAM_BOTTOM            EQU         0x30000000
SDRAM_TOP               EQU         0x33ffffff      
;------------------------------------------------------------------    
MMU_MASTER_PT_BASE      EQU         SDRAM_TOP-0x3fff   ;be not used
;------------------------------------------------------------------
STACK_BASE              EQU         MMU_MASTER_PT_BASE-0xc000
STACK_SVC               EQU         STACK_BASE          ;64KB 
STACK_IRQ               EQU         STACK_SVC-0x10000   ;64KB 
STACK_FIQ               EQU         STACK_IRQ-0x10000   ;64KB 
STACK_ABORT             EQU         STACK_FIQ-0x10000   ;64KB 
STACK_UNDEF             EQU         STACK_ABORT-0x10000 ;64KB 
STACK_USER              EQU         STACK_UNDEF-0x10000 ;64KB 
STACK_LIMIT             EQU         STACK_USER-0x10000  ;     

HEAP_SIZE               EQU         0x800000           ;32MB
HEAP_BASE               EQU         SDRAM_BOTTOM+0x1000000  
HEAP_LIMIT              EQU         HEAP_BASE+HEAP_SIZE      

;-------------------------------------------------------------
; -- the constant below are not used
C_LIB_STATIC_BASE       EQU         STACK_LIMIT-0x10000      
FP_OFSET                EQU         0x04
FP_STATUS_ADDR          EQU         C_LIB_STATIC_BASE-FP_OFSET
;------------------------------------------------------------- 

;---------------------------------------------------------------
; -- Assemble options
    GBLL    LOAD_BY_SUPERVIVI
LOAD_BY_SUPERVIVI           SETL        {FALSE}     ;be not used
;---------------------------------------------------------------

; -- Memory controller configuration datas
;--------------------------------------------------------------------------
; -- Bank map for mini2440 ...
; Bank 0            - It depend on the logic level of OM[1:0] at reset
; Bank 1            - FPGA(Signal Acquisition And Communication Board)
; Bank 4            - CS9000                    @ 16bits
; Bank 6            - SDRAM                     @ 32bits
; Bank 7            - unconnect
; Bank 2 3 5      - reserved for extension   
;--------------------------------------------------------------------------
; -- BUSCON parameter
DW8             EQU	        (0x00<<0)
DW16            EQU	        (0x01<<0)
DW32            EQU     	(0x02<<0)
WAIT            EQU     	(0x01<<2)
UBLB            EQU         (0x01<<3)

B1_BWSCON       EQU         (DW32<<4)
B2_BWSCON       EQU         (DW16<<8)
B3_BWSCON       EQU         (DW16<<12)
B4_BWSCON       EQU         (DW16<<16)
B5_BWSCON       EQU         (DW16<<20)
B6_BWSCON       EQU         (DW32<<24)
B7_BWSCON       EQU         (DW32<<28)
; -- Bank 0 parameter
B0_Tacs         EQU         (0x03<<13)
B0_Tcos         EQU         (0x03<<11)
B0_Tacc         EQU         (0x07<<8)
B0_Tcoh         EQU         (0x03<<6)
B0_Tah          EQU         (0x03<<4)
B0_Tacp         EQU         (0x01<<2)
B0_PMC          EQU         (0x00<<0)
; -- Bank 1 parameter
B1_Tacs         EQU         (0x00<<13)
B1_Tcos         EQU         (0x00<<11)
B1_Tacc         EQU         (0x06<<8)
B1_Tcoh         EQU         (0x00<<6)
B1_Tah          EQU         (0x00<<4)
B1_Tacp         EQU         (0x00<<2)
B1_PMC          EQU         (0x00<<0)
; -- Bank 2 parameter
B2_Tacs         EQU         (0x01<<13)
B2_Tcos         EQU         (0x01<<11)
B2_Tacc         EQU         (0x04<<8)
B2_Tcoh         EQU         (0x01<<6)
B2_Tah          EQU         (0x01<<4)
B2_Tacp         EQU         (0x00<<2)
B2_PMC          EQU         (0x00<<0)
; -- Bank 3 parameter
B3_Tacs         EQU         (0x01<<13)
B3_Tcos         EQU         (0x01<<11)
B3_Tacc         EQU         (0x04<<8)
B3_Tcoh         EQU         (0x01<<6)
B3_Tah          EQU         (0x01<<4)
B3_Tacp         EQU         (0x00<<2)
B3_PMC          EQU         (0x00<<0)
; -- Bank 4 parameter
B4_Tacs         EQU         (0x01<<13)
B4_Tcos         EQU         (0x01<<11)
B4_Tacc         EQU         (0x04<<8)
B4_Tcoh         EQU         (0x01<<6)
B4_Tah          EQU         (0x01<<4)
B4_Tacp         EQU         (0x00<<2)
B4_PMC          EQU         (0x00<<0)
; -- Bank 5 parameter
B5_Tacs         EQU         (0x01<<13)
B5_Tcos         EQU         (0x01<<11)
B5_Tacc         EQU         (0x04<<8)
B5_Tcoh         EQU         (0x01<<6)
B5_Tah          EQU         (0x01<<4)
B5_Tacp         EQU         (0x00<<2)
B5_PMC          EQU         (0x00<<1)

; -- Bank 6 parameter
; -- Just for HY57V561620FTP-H
; -- !!!HCLK = 101.25MHz
B6_MT           EQU         (0x03<<15)
B6_Trcd         EQU         (0x01<<2)   ;3 clocks
B6_SCAN         EQU         (0x01<<0)
; -- Bank 7 parameter
B7_MT           EQU         (0x03<<15)	
B7_Trcd         EQU         (0x01<<2)   ;3 clocks
B7_SCAN         EQU         (0x01<<0)
; -- REFRESH parameter
REFEN           EQU         (0x01<<23)  ;refresh enable 
TREFMD          EQU         (0x00<<22)  ;cbr/auto refresh
Trp             EQU         (0x01<<20)  ;3 clocks
Tsrc            EQU         (0x02<<18)  ;6 clocks
REFCNT          EQU         (1258<<0)   ;2^11 + 1 - 101.25 * 7.8125
; -- BANKSIZE parameter 
BURST           EQU         (0x01<<7)
SCKE            EQU         (0x00<<5)
SCLK            EQU         (0x00<<4)
BKM76MAP        EQU         (0x01<<0)
; -- MRSRB6&7 parameter
WBL             EQU         (0x00<<9)
TM              EQU         (0x00<<7)
CL              EQU         (0x02<<4)   ;2 clocks, it should be 3 clocks 
BT              EQU         (0x00<<3)   ;but it works well, so...
BL              EQU         (0x00<<0)

; -- get entry of interrupt handler from vectors jump table
    MACRO
$A HANDLER $B
$A  SUB sp, sp, #4
    STMFD sp!, {r0}
    LDR r0, =$B
    LDR r0, [r0]
    STR r0, [sp, #0x4]
    LDMFD sp!, {r0, pc}
    MEND
    
    END

