;--------------------------------------------------------------------------
;Copyright(c) 2009, WenZhong Sun<wanmyqawdr@126.com>
;--------------------------------------------------------------------------
;This file is part of ARMV4T_and_FPGA_based_DSO

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either
;version 2 of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; -- system_initialize is reset handler that first execute when processor
; -- power up. This handler do some necessary initializtions such as setup 
; -- stacks for each processor mode, heaps for dynamic memory allocation, 
; -- system clock, peripherals, C LIB... then jump to C code entry -- main.
;--------------------------------------------------------------------------
    
    GET ./options.inc
    GET ./S3C2440regs.inc
    

    AREA SYSTEM_INITIALIZE, CODE, READONLY
    
; -- CODE

;--------------------------------------------------------------------------
; -- system_initialize   
    EXPORT system_initialize
system_initialize
    BL disable_watchdog            ;Disable watchdog timer
    BL mask_interrupts             ;Mask all interrupt sources
    BL configure_system_clock      ;Set initial clock for core and peripherals
    BL configure_memory_controller ;Configure memory controller 
   ; IMPORT fill_vectors_table      ;
   ; BL fill_vectors_table          ;Write interrupts table to a higher memory region 
    BL setup_stacks                ;Setup stacks for each processor mode
    
; -- Here use __main in C LIB, it will copy RW, RO datas from load region to
; -- execution region, initialize ZI region and C LIB, then jump to C code entry
; -- You can also simply do this by yourself. But you should invoke __main instead 
; -- if you use a scatter file to realize some complex image arrangement.

    IMPORT __main   
    b __main                  ;not BL, because application never return this way
    
; -- system_initialize end 
;---------------------------------------------------------------------------

disable_watchdog
    LDR r0, =WTCON
    LDR r1, =0x0
    STR r1, [r0]
    MOV pc, lr
   
mask_interrupts
    LDR r0, =INTMSK
    LDR r1, =0xffffffff
    STR r1, [r0]
    LDR r0, =INTSUBMSK
    STR r1, [r0]
    MOV pc, lr

configure_system_clock
; -- First reduce pll lock time
    LDR r0, =LOCKTIME
    LDR r1, =0xffffffff
    STR r1, [r0]; 
; -- Configure frequency proportional    
    LDR r0, =CLKDIVN
    LDR r1, =CLKDIVN_VAL
    STR r1, [r0]
; -- Configure UPLL    
    LDR r0, =UPLLCON
    LDR r1, =U_MDIV|U_PDIV|U_SDIV
    STR r1, [r0]
; -- Wait 7 nops before stabilization of UPLL    
    NOP
    NOP
    NOP
    NOP
    NOP
    NOP
    NOP
; -- Configure MPLL    
    LDR r0, =MPLLCON
    LDR r1, =M_MDIV|M_PDIV|M_SDIV
    STR r1, [r0]
; -- The crystal @ 12MHz
; -- !!! Now FCLK = 405MHz, HCLK = 101.25MHz, PCLK = 50.625MHz, UCLK = 48MHz

; -- Should change to asynchronous bus mode because FCLK != HCLK
; -- See TRM of ARM920T and datasheet of S3C2440 for details
    IMPORT _cp15_SetAsyncBusMode  
    B _cp15_SetAsyncBusMode

configure_memory_controller
    IMPORT MEMORY_CONTROLLER_CONFIGURATION_DATA
    LDR r0, =MEMORY_CONTROLLER_CONFIGURATION_DATA
    LDR r1, =BWSCON
    ADD r2, r0, #0x34   ;The last register at 0x48000030
1
    LDR r3, [r0], #4
    STR r3, [r1], #4
    CMP r2, r0
    BNE %B1
    MOV pc, lr
    
setup_stacks
    MOV r4, lr
    MRS r0, cpsr                   
    
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(ABORT_MODE|NOINT)
    MSR cpsr_c, r0
    LDR sp, =STACK_ABORT           
    
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(FIQ_MODE|NOINT)
    MSR cpsr_c, r0
    LDR sp, =STACK_FIQ             
    
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(IRQ_MODE|NOINT)
    MSR cpsr_c, r0
    LDR sp, =STACK_IRQ                  
        
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(UNDEF_MODE|NOINT)
    MSR cpsr_c, r0
    LDR sp, =STACK_UNDEF    
 
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(SVC_MODE|NOINT)
    MSR cpsr_c, r0
    LDR sp, =STACK_SVC             
    
    
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(SYS_MODE|NOINT)
    MSR cpsr_c, r0
;    LDR sp, =STACK_USER     ;__main() will do it
    MOV pc, r4
    
    EXPORT _enter_user_mode
_enter_user_mode    
;enter user mode and enable IRQ
    MRS r0, cpsr
    BIC r0, r0, #MODE_MASK
    ORR r0, r0, #(USER_MODE|NOINT)
    BIC r0, r0, #I_BIT
    MSR cpsr_c, r0
    MOV pc, lr
    
    END
